# Copyright 2016 VMware, Inc.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import sys

from vmware_nsx.common import utils as nsx_utils
from vmware_nsx.db import db as nsx_db
from vmware_nsx.shell.admin.plugins.common import constants
from vmware_nsx.shell.admin.plugins.common import formatters
from vmware_nsx.shell.admin.plugins.common import utils as admin_utils
from vmware_nsx.shell.admin.plugins.nsxv3.resources import utils
from vmware_nsx.shell import resources as shell
from vmware_nsxlib.v3 import exceptions as nsx_exc
from vmware_nsxlib.v3 import nsx_constants

from neutron.db import db_base_plugin_v2
from neutron.db import l3_db
from neutron_lib.callbacks import registry
from neutron_lib import context as neutron_context
from oslo_log import log as logging

LOG = logging.getLogger(__name__)
neutron_client = utils.NeutronDbClient()


class RoutersPlugin(db_base_plugin_v2.NeutronDbPluginV2,
                    l3_db.L3_NAT_db_mixin):
    pass


@admin_utils.output_header
def list_missing_routers(resource, event, trigger, **kwargs):
    """List neutron routers that are missing the NSX backend router
    """
    nsxlib = utils.get_connected_nsxlib()
    plugin = RoutersPlugin()
    admin_cxt = neutron_context.get_admin_context()
    filters = utils.get_plugin_filters(admin_cxt)
    neutron_routers = plugin.get_routers(admin_cxt, filters=filters)
    routers = []
    for router in neutron_routers:
        neutron_id = router['id']
        # get the router nsx id from the mapping table
        nsx_id = nsx_db.get_nsx_router_id(admin_cxt.session,
                                          neutron_id)
        if not nsx_id:
            routers.append({'name': router['name'],
                            'neutron_id': neutron_id,
                            'nsx_id': None})
        else:
            try:
                nsxlib.logical_router.get(nsx_id)
            except nsx_exc.ResourceNotFound:
                routers.append({'name': router['name'],
                              'neutron_id': neutron_id,
                              'nsx_id': nsx_id})
    if len(routers) > 0:
        title = ("Found %d routers missing from the NSX "
                 "manager:") % len(routers)
        LOG.info(formatters.output_formatter(
            title, routers,
            ['name', 'neutron_id', 'nsx_id']))
    else:
        LOG.info("All routers exist on the NSX manager")


@admin_utils.output_header
def update_nat_rules(resource, event, trigger, **kwargs):
    """Update all routers NAT rules to not bypass the firewall"""
    # This feature is supported only since nsx version 2
    nsxlib = utils.get_connected_nsxlib()
    version = nsxlib.get_version()
    if not nsx_utils.is_nsx_version_2_0_0(version):
        LOG.info("NAT rules update only supported from 2.0 onwards")
        LOG.info("Version is %s", version)
        return

    # Go over all neutron routers
    plugin = RoutersPlugin()
    admin_cxt = neutron_context.get_admin_context()
    filters = utils.get_plugin_filters(admin_cxt)
    neutron_routers = plugin.get_routers(admin_cxt, filters=filters)
    num_of_updates = 0
    for router in neutron_routers:
        neutron_id = router['id']
        # get the router nsx id from the mapping table
        nsx_id = nsx_db.get_nsx_router_id(admin_cxt.session,
                                          neutron_id)
        if nsx_id:
            # get all NAT rules:
            rules = nsxlib.logical_router.list_nat_rules(nsx_id)['results']
            for rule in rules:
                if 'nat_pass' not in rule or rule['nat_pass']:
                    nsxlib.logical_router.update_nat_rule(
                        nsx_id, rule['id'], nat_pass=False)
                    num_of_updates = num_of_updates + 1
    if num_of_updates:
        LOG.info("Done updating %s NAT rules", num_of_updates)
    else:
        LOG.info("Did not find any NAT rule to update")


@admin_utils.output_header
def list_orphaned_routers(resource, event, trigger, **kwargs):
    nsxlib = utils.get_connected_nsxlib()
    nsx_routers = nsxlib.logical_router.list()['results']
    missing_routers = []
    for nsx_router in nsx_routers:
        # check if it exists in the neutron DB
        if not neutron_client.lrouter_id_to_router_id(nsx_router['id']):
            # Skip non-neutron routers, by tags
            for tag in nsx_router.get('tags', []):
                if tag.get('scope') == 'os-neutron-router-id':
                    missing_routers.append(nsx_router)
                    break

    LOG.info(formatters.output_formatter(constants.ORPHANED_ROUTERS,
                                         missing_routers,
                                         ['id', 'display_name']))


@admin_utils.output_header
def delete_backend_router(resource, event, trigger, **kwargs):
    nsxlib = utils.get_connected_nsxlib()
    errmsg = ("Need to specify nsx-id property. Add --property nsx-id=<id>")
    if not kwargs.get('property'):
        LOG.error("%s", errmsg)
        return
    properties = admin_utils.parse_multi_keyval_opt(kwargs['property'])
    nsx_id = properties.get('nsx-id')
    if not nsx_id:
        LOG.error("%s", errmsg)
        return

    # check if the router exists
    try:
        nsxlib.logical_router.get(nsx_id, silent=True)
    except nsx_exc.ResourceNotFound:
        # prevent logger from logging this exception
        sys.exc_clear()
        LOG.warning("Backend router %s was not found.", nsx_id)
        return

    # try to delete it
    try:
        # first delete its ports
        ports = nsxlib.logical_router_port.get_by_router_id(nsx_id)
        for port in ports:
            nsxlib.logical_router_port.delete(port['id'])
        nsxlib.logical_router.delete(nsx_id)
    except Exception as e:
        LOG.error("Failed to delete backend router %(id)s : %(e)s.", {
            'id': nsx_id, 'e': e})
        return

    # Verify that the router was deleted since the backend does not always
    # throws errors
    try:
        nsxlib.logical_router.get(nsx_id, silent=True)
    except nsx_exc.ResourceNotFound:
        # prevent logger from logging this exception
        sys.exc_clear()
        LOG.info("Backend router %s was deleted.", nsx_id)
    else:
        LOG.error("Failed to delete backend router %s.", nsx_id)


@admin_utils.output_header
def update_dhcp_relay(resource, event, trigger, **kwargs):
    """Update all routers dhcp relay service by the current configuration"""
    nsxlib = utils.get_connected_nsxlib()
    if not nsxlib.feature_supported(nsx_constants.FEATURE_DHCP_RELAY):
        version = nsxlib.get_version()
        LOG.error("DHCP relay is not supported by NSX version %s", version)
        return

    admin_cxt = neutron_context.get_admin_context()
    filters = utils.get_plugin_filters(admin_cxt)
    with utils.NsxV3PluginWrapper() as plugin:
        # Make sure FWaaS was initialized
        plugin.init_fwaas_for_admin_utils()

        # get all neutron routers and  interfaces ports
        routers = plugin.get_routers(admin_cxt, filters=filters)
        for router in routers:
            LOG.info("Updating router %s", router['id'])
            port_filters = {'device_owner': [l3_db.DEVICE_OWNER_ROUTER_INTF],
                            'device_id': [router['id']]}
            ports = plugin.get_ports(admin_cxt, filters=port_filters)
            for port in ports:
                # get the backend router port by the tag
                nsx_port_id = nsxlib.get_id_by_resource_and_tag(
                    'LogicalRouterDownLinkPort',
                    'os-neutron-rport-id', port['id'])
                if not nsx_port_id:
                    LOG.warning("Couldn't find nsx router port for interface "
                                "%s", port['id'])
                    continue
                # get the network of this port
                network_id = port['network_id']
                # check the relay service on the az of the network
                az = plugin.get_network_az_by_net_id(admin_cxt, network_id)
                nsxlib.logical_router_port.update(
                    nsx_port_id, relay_service_uuid=az.dhcp_relay_service)

            # if FWaaS is enables, also update the firewall rules
            try:
                plugin.update_router_firewall(admin_cxt, router['id'])
            except Exception:
                pass

    LOG.info("Done.")


registry.subscribe(list_missing_routers,
                   constants.ROUTERS,
                   shell.Operations.LIST_MISMATCHES.value)

registry.subscribe(update_nat_rules,
                   constants.ROUTERS,
                   shell.Operations.NSX_UPDATE_RULES.value)

registry.subscribe(list_orphaned_routers,
                   constants.ORPHANED_ROUTERS,
                   shell.Operations.LIST.value)

registry.subscribe(delete_backend_router,
                   constants.ORPHANED_ROUTERS,
                   shell.Operations.NSX_CLEAN.value)

registry.subscribe(update_dhcp_relay,
                   constants.ROUTERS,
                   shell.Operations.NSX_UPDATE_DHCP_RELAY.value)



# Copyright 2017 VMware, Inc.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from neutron_lib.callbacks import events
from neutron_lib.callbacks import registry
from neutron_lib.callbacks import resources
from neutron_lib import exceptions as n_exc
from oslo_log import helpers as log_helpers
from oslo_log import log as logging
from oslo_utils import excutils

from vmware_nsx._i18n import _
from vmware_nsx.db import db as nsx_db
from vmware_nsx.services.lbaas import base_mgr
from vmware_nsx.services.lbaas import lb_const
from vmware_nsx.services.lbaas.nsx_v3 import lb_utils
from vmware_nsxlib.v3 import exceptions as nsxlib_exc
from vmware_nsxlib.v3 import utils

LOG = logging.getLogger(__name__)


class EdgeLoadBalancerManager(base_mgr.Nsxv3LoadbalancerBaseManager):
    @log_helpers.log_method_call
    def __init__(self):
        super(EdgeLoadBalancerManager, self).__init__()
        registry.subscribe(
            self._handle_subnet_gw_change,
            resources.SUBNET, events.AFTER_UPDATE)

    @log_helpers.log_method_call
    def create(self, context, lb):
        if lb_utils.validate_lb_subnet(context, self.core_plugin,
                                       lb.vip_subnet_id):
            self.lbv2_driver.load_balancer.successful_completion(context, lb)
        else:
            msg = (_('Cannot create lb on subnet %(sub)s for '
                     'loadbalancer %(lb)s. The subnet needs to connect a '
                     'router which is already set gateway.') %
                   {'sub': lb.vip_subnet_id, 'lb': lb.id})
            raise n_exc.BadRequest(resource='lbaas-subnet', msg=msg)

    @log_helpers.log_method_call
    def update(self, context, old_lb, new_lb):
        vs_client = self.core_plugin.nsxlib.load_balancer.virtual_server
        app_client = self.core_plugin.nsxlib.load_balancer.application_profile
        if new_lb.name != old_lb.name:
            for listener in new_lb.listeners:
                binding = nsx_db.get_nsx_lbaas_listener_binding(
                    context.session, new_lb.id, listener.id)
                if binding:
                    vs_id = binding['lb_vs_id']
                    app_profile_id = binding['app_profile_id']
                    new_lb_name = new_lb.name[:utils.MAX_TAG_LEN]
                    try:
                        # Update tag on virtual server with new lb name
                        vs = vs_client.get(vs_id)
                        updated_tags = utils.update_v3_tags(
                            vs['tags'], [{'scope': lb_const.LB_LB_NAME,
                                          'tag': new_lb_name}])
                        vs_client.update(vs_id, tags=updated_tags)
                        # Update tag on application profile with new lb name
                        app_profile = app_client.get(app_profile_id)
                        app_client.update(
                            app_profile_id, tags=updated_tags,
                            resource_type=app_profile['resource_type'])

                    except nsxlib_exc.ManagerError:
                        with excutils.save_and_reraise_exception():
                            self.lbv2_driver.pool.failed_completion(context,
                                                                    new_lb)
                            LOG.error('Failed to update tag %(tag)s for lb '
                                      '%(lb)s', {'tag': updated_tags,
                                                 'lb': new_lb.name})

        self.lbv2_driver.load_balancer.successful_completion(context, new_lb)

    @log_helpers.log_method_call
    def delete(self, context, lb):
        service_client = self.core_plugin.nsxlib.load_balancer.service
        lb_binding = nsx_db.get_nsx_lbaas_loadbalancer_binding(
            context.session, lb.id)
        if lb_binding:
            lb_service_id = lb_binding['lb_service_id']
            nsx_router_id = lb_binding['lb_router_id']
            try:
                lb_service = service_client.get(lb_service_id)
            except nsxlib_exc.ManagerError:
                LOG.warning("LB service %(lbs)s is not found",
                            {'lbs': lb_service_id})
            else:
                vs_list = lb_service.get('virtual_server_ids')
                if not vs_list:
                    try:
                        service_client.delete(lb_service_id)
                        # If there is no lb service attached to the router,
                        # update the router advertise_lb_vip flag to false.
                        router_client = self.core_plugin.nsxlib.logical_router
                        router_client.update_advertisement(
                            nsx_router_id, advertise_lb_vip=False)
                    except nsxlib_exc.ManagerError:
                        self.lbv2_driver.load_balancer.failed_completion(
                            context, lb, delete=True)
                        msg = (_('Failed to delete lb service %(lbs)s from nsx'
                                 ) % {'lbs': lb_service_id})
                        raise n_exc.BadRequest(resource='lbaas-lb', msg=msg)
            nsx_db.delete_nsx_lbaas_loadbalancer_binding(
                context.session, lb.id)
        self.lbv2_driver.load_balancer.successful_completion(
            context, lb, delete=True)

    @log_helpers.log_method_call
    def refresh(self, context, lb):
        # TODO(tongl): implememnt
        pass

    @log_helpers.log_method_call
    def stats(self, context, lb):
        # Since multiple LBaaS loadbalancer can share the same LB service,
        # get the corresponding virtual servers' stats instead of LB service.
        stats = {'active_connections': 0,
                 'bytes_in': 0,
                 'bytes_out': 0,
                 'total_connections': 0}

        service_client = self.core_plugin.nsxlib.load_balancer.service
        lb_binding = nsx_db.get_nsx_lbaas_loadbalancer_binding(
            context.session, lb.id)
        vs_list = self._get_lb_virtual_servers(context, lb)
        if lb_binding:
            lb_service_id = lb_binding.get('lb_service_id')
            try:
                rsp = service_client.get_stats(lb_service_id)
                if rsp:
                    for vs in rsp['virtual_servers']:
                        # Skip the virtual server that doesn't belong
                        # to this loadbalancer
                        if vs['virtual_server_id'] not in vs_list:
                            continue
                        vs_stats = vs['statistics']
                        for stat in lb_const.LB_STATS_MAP:
                            lb_stat = lb_const.LB_STATS_MAP[stat]
                            stats[stat] += vs_stats[lb_stat]

            except nsxlib_exc.ManagerError:
                msg = _('Failed to retrieve stats from LB service '
                        'for loadbalancer %(lb)s') % {'lb': lb.id}
                raise n_exc.BadRequest(resource='lbaas-lb', msg=msg)
        return stats

    def _get_lb_virtual_servers(self, context, lb):
        # Get all virtual servers that belong to this loadbalancer
        vs_list = []
        for listener in lb.listeners:
            vs_binding = nsx_db.get_nsx_lbaas_listener_binding(
                context.session, lb.id, listener.id)
            if vs_binding:
                vs_list.append(vs_binding.get('lb_vs_id'))
        return vs_list

    def _handle_subnet_gw_change(self, *args, **kwargs):
        # As the Edge appliance doesn't use DHCP, we should change the
        # default gateway here when the subnet GW changes.
        orig = kwargs['original_subnet']
        updated = kwargs['subnet']
        if orig['gateway_ip'] == updated['gateway_ip']:
            return



# Copyright (c) 2014 VMware.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock
from oslo_config import cfg
from oslo_vmware import api

from neutron.tests import base
from vmware_nsx.dvs import dvs_utils


class DvsUtilsTestCase(base.BaseTestCase):

    def test_default_configuration(self):
        self.assertFalse(dvs_utils.dvs_is_enabled())

    def _dvs_fake_cfg_set(self):
        cfg.CONF.set_override('host_ip', 'fake_host_ip',
                              group='dvs')
        cfg.CONF.set_override('host_username', 'fake_host_user_name',
                              group='dvs')
        cfg.CONF.set_override('host_password', 'fake_host_pasword',
                              group='dvs')
        cfg.CONF.set_override('dvs_name', 'fake_dvs', group='dvs')
        cfg.CONF.set_override('host_port', '443', group='dvs')
        cfg.CONF.set_override('ca_file', 'cacert', group='dvs')
        cfg.CONF.set_override('insecure', False, group='dvs')

    def test_dvs_set(self):
        self._dvs_fake_cfg_set()
        self.assertTrue(dvs_utils.dvs_is_enabled())

    @mock.patch.object(api.VMwareAPISession, '__init__',
                       return_value=None)
    def test_dvs_create_session(self, fake_init):
        dvs_utils.dvs_create_session()
        fake_init.assert_called_once_with(cfg.CONF.dvs.host_ip,
                                          cfg.CONF.dvs.host_username,
                                          cfg.CONF.dvs.host_password,
                                          cfg.CONF.dvs.api_retry_count,
                                          cfg.CONF.dvs.task_poll_interval,
                                          port=cfg.CONF.dvs.host_port,
                                          cacert=cfg.CONF.dvs.ca_file,
                                          insecure=cfg.CONF.dvs.insecure)

    def test_dvs_name_get(self):
        cfg.CONF.set_override('dvs_name', 'fake-dvs', group='dvs')
        self.assertEqual('fake-dvs',
                         dvs_utils.dvs_name_get())



# Copyright (c) 2014 OpenStack Foundation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib.api.definitions import allowedaddresspairs as addr_apidef
from neutron_lib.api.definitions import port_security as psec
from oslo_config import cfg

from neutron.tests.unit.db import test_allowedaddresspairs_db as ext_pairs

from vmware_nsx.tests.unit.nsx_mh import test_plugin as test_nsx_plugin
from vmware_nsx.tests.unit.nsx_v import test_plugin as test_nsx_v_plugin
from vmware_nsx.tests.unit.nsx_v3 import test_constants as v3_constants
from vmware_nsx.tests.unit.nsx_v3 import test_plugin as test_v3_plugin


class TestAllowedAddressPairsNSXv2(test_nsx_plugin.NsxPluginV2TestCase,
                                   ext_pairs.TestAllowedAddressPairs):

    # TODO(arosen): move to ext_pairs.TestAllowedAddressPairs once all
    # plugins do this correctly.
    def test_create_port_no_allowed_address_pairs(self):
        with self.network() as net:
            res = self._create_port(self.fmt, net['network']['id'])
            port = self.deserialize(self.fmt, res)
            self.assertEqual(port['port'][addr_apidef.ADDRESS_PAIRS], [])
            self._delete('ports', port['port']['id'])

    def test_create_port_security_false_allowed_address_pairs(self):
        self.skipTest('TBD')


class TestAllowedAddressPairsNSXv3(test_v3_plugin.NsxV3PluginTestCaseMixin,
                                   ext_pairs.TestAllowedAddressPairs):

    def setUp(self, plugin=v3_constants.PLUGIN_NAME,
              ext_mgr=None,
              service_plugins=None):
        super(TestAllowedAddressPairsNSXv3, self).setUp(
            plugin=plugin, ext_mgr=ext_mgr, service_plugins=service_plugins)

    def test_create_bad_address_pairs_with_cidr(self):
        address_pairs = [{'mac_address': '00:00:00:00:00:01',
                          'ip_address': '10.0.0.1/24'}]
        self._create_port_with_address_pairs(address_pairs, 400)

    def test_update_add_bad_address_pairs_with_cidr(self):
        with self.network() as net:
            res = self._create_port(self.fmt, net['network']['id'])
            port = self.deserialize(self.fmt, res)
            address_pairs = [{'mac_address': '00:00:00:00:00:01',
                              'ip_address': '10.0.0.1/24'}]
            update_port = {'port': {addr_apidef.ADDRESS_PAIRS:
                                    address_pairs}}
            req = self.new_update_request('ports', update_port,
                                          port['port']['id'])
            res = req.get_response(self.api)
            self.assertEqual(res.status_int, 400)
            self._delete('ports', port['port']['id'])

    def test_create_port_security_false_allowed_address_pairs(self):
        self.skipTest('TBD')


class TestAllowedAddressPairsNSXv(test_nsx_v_plugin.NsxVPluginV2TestCase,
                                  ext_pairs.TestAllowedAddressPairs):

    def setUp(self, plugin='vmware_nsx.plugin.NsxVPlugin',
              ext_mgr=None,
              service_plugins=None):
        super(TestAllowedAddressPairsNSXv, self).setUp(
            plugin=plugin, ext_mgr=ext_mgr, service_plugins=service_plugins)

    def test_create_port_security_false_allowed_address_pairs(self):
        self.skipTest('TBD')

    def test_update_port_security_off_address_pairs(self):
        self.skipTest('Not supported')

    def test_create_overlap_with_fixed_ip(self):
        address_pairs = [{'ip_address': '10.0.0.2'}]
        with self.network() as network:
            with self.subnet(network=network, cidr='10.0.0.0/24',
                             enable_dhcp=False) as subnet:
                fixed_ips = [{'subnet_id': subnet['subnet']['id'],
                              'ip_address': '10.0.0.2'}]
                res = self._create_port(self.fmt, network['network']['id'],
                                        arg_list=(addr_apidef.ADDRESS_PAIRS,
                                        'fixed_ips'),
                                        allowed_address_pairs=address_pairs,
                                        fixed_ips=fixed_ips)
                self.assertEqual(res.status_int, 201)
                port = self.deserialize(self.fmt, res)
                self._delete('ports', port['port']['id'])

    def test_create_port_allowed_address_pairs(self):
        with self.network() as net:
            address_pairs = [{'ip_address': '10.0.0.1'}]
            res = self._create_port(self.fmt, net['network']['id'],
                                    arg_list=(addr_apidef.ADDRESS_PAIRS,),
                                    allowed_address_pairs=address_pairs)
            port = self.deserialize(self.fmt, res)
            address_pairs[0]['mac_address'] = port['port']['mac_address']
            self.assertEqual(port['port'][addr_apidef.ADDRESS_PAIRS],
                             address_pairs)
            self._delete('ports', port['port']['id'])

    def _test_create_port_remove_allowed_address_pairs(self, update_value):
        with self.network() as net:
            address_pairs = [{'ip_address': '10.0.0.1'}]
            res = self._create_port(self.fmt, net['network']['id'],
                                    arg_list=(addr_apidef.ADDRESS_PAIRS,),
                                    allowed_address_pairs=address_pairs)
            port = self.deserialize(self.fmt, res)
            update_port = {'port': {addr_apidef.ADDRESS_PAIRS: []}}
            req = self.new_update_request('ports', update_port,
                                          port['port']['id'])
            port = self.deserialize(self.fmt, req.get_response(self.api))
            self.assertEqual(port['port'][addr_apidef.ADDRESS_PAIRS], [])
            self._delete('ports', port['port']['id'])

    def test_update_add_address_pairs(self):
        with self.network() as net:
            res = self._create_port(self.fmt, net['network']['id'])
            port = self.deserialize(self.fmt, res)
            address_pairs = [{'ip_address': '10.0.0.1'}]
            update_port = {'port': {addr_apidef.ADDRESS_PAIRS:
                                    address_pairs}}
            req = self.new_update_request('ports', update_port,
                                          port['port']['id'])
            port = self.deserialize(self.fmt, req.get_response(self.api))
            address_pairs[0]['mac_address'] = port['port']['mac_address']
            self.assertEqual(port['port'][addr_apidef.ADDRESS_PAIRS],
                             address_pairs)
            self._delete('ports', port['port']['id'])

    def test_mac_configuration(self):
        address_pairs = [{'mac_address': '00:00:00:00:00:01',
                          'ip_address': '10.0.0.1'}]
        self._create_port_with_address_pairs(address_pairs, 400)

    def test_equal_to_max_allowed_address_pair(self):
        cfg.CONF.set_default('max_allowed_address_pair', 3)
        address_pairs = [{'ip_address': '10.0.0.1'},
                         {'ip_address': '10.0.0.2'},
                         {'ip_address': '10.0.0.3'}]
        self._create_port_with_address_pairs(address_pairs, 201)

    def test_create_port_security_true_allowed_address_pairs(self):
        if self._skip_port_security:
            self.skipTest("Plugin does not implement port-security extension")

        with self.network() as net:
            address_pairs = [{'ip_address': '10.0.0.1'}]
            res = self._create_port(self.fmt, net['network']['id'],
                                    arg_list=('port_security_enabled',
                                              addr_apidef.ADDRESS_PAIRS,),
                                    port_security_enabled=True,
                                    allowed_address_pairs=address_pairs)
            port = self.deserialize(self.fmt, res)
            self.assertTrue(port['port'][psec.PORTSECURITY])
            address_pairs[0]['mac_address'] = port['port']['mac_address']
            self.assertEqual(port['port'][addr_apidef.ADDRESS_PAIRS],
                             address_pairs)
            self._delete('ports', port['port']['id'])



# Copyright 2016 VMware, Inc.
# All Rights Reserved
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from oslo_config import cfg

from vmware_nsx.tests.unit.nsx_v import test_plugin

from neutron_lib.api.definitions import provider_net as pnet


class TestNsxvIpamSubnets(test_plugin.TestSubnetsV2):
    """Run the nsxv plugin subnets tests with the ipam driver"""
    def setUp(self):
        cfg.CONF.set_override(
            "ipam_driver",
            "vmware_nsx.services.ipam.nsx_v.driver.NsxvIpamDriver")
        super(TestNsxvIpamSubnets, self).setUp()

    def provider_net(self):
        name = 'dvs-provider-net'
        providernet_args = {pnet.NETWORK_TYPE: 'vlan',
                            pnet.SEGMENTATION_ID: 43,
                            pnet.PHYSICAL_NETWORK: 'dvs-uuid'}
        return self.network(name=name, do_delete=False,
                            providernet_args=providernet_args,
                            arg_list=(pnet.NETWORK_TYPE,
                                      pnet.SEGMENTATION_ID,
                                      pnet.PHYSICAL_NETWORK))

    def test_provider_net_use_driver(self):
        with self.provider_net() as net:
            before = len(self.fc2._ipam_pools)
            with self.subnet(network=net, cidr='10.10.10.0/29',
                             enable_dhcp=False):
                self.assertEqual(before + 1, len(self.fc2._ipam_pools))

    def test_ext_net_use_driver(self):
        with self.network(router__external=True) as net:
            before = len(self.fc2._ipam_pools)
            with self.subnet(network=net, cidr='10.10.10.0/29',
                             enable_dhcp=False):
                self.assertEqual(before + 1, len(self.fc2._ipam_pools))

    def test_regular_net_dont_use_driver(self):
        with self.network() as net:
            before = len(self.fc2._ipam_pools)
            with self.subnet(network=net, cidr='10.10.10.0/29',
                             enable_dhcp=False):
                self.assertEqual(before, len(self.fc2._ipam_pools))

    def test_no_more_ips(self):
        # create a small provider network, and use all the IPs
        with self.provider_net() as net:
            with self.subnet(network=net, cidr='10.10.10.0/29',
                             enable_dhcp=False) as subnet:
                # create ports on this subnet until there are no more free ips
                # legal ips are 10.10.10.2 - 10.10.10.6
                fixed_ips = [{'subnet_id': subnet['subnet']['id']}]
                for counter in range(5):
                    port_res = self._create_port(
                        self.fmt, net['network']['id'], fixed_ips=fixed_ips)
                    port = self.deserialize('json', port_res)
                    self.assertIn('port', port)

                # try to create another one - should fail
                port_res = self._create_port(
                    self.fmt, net['network']['id'], fixed_ips=fixed_ips)
                port = self.deserialize('json', port_res)
                self.assertIn('NeutronError', port)
                self.assertIn('message', port['NeutronError'])
                self.assertTrue(('No more IP addresses available' in
                                 port['NeutronError']['message']))

    def test_use_same_ips(self):
        # create a provider network and try to allocate the same ip twice
        with self.provider_net() as net:
            with self.subnet(network=net, cidr='10.10.10.0/24',
                             enable_dhcp=False) as subnet:
                fixed_ips = [{'ip_address': '10.10.10.2',
                              'subnet_id': subnet['subnet']['id']}]
                # First port should succeed
                port_res = self._create_port(
                    self.fmt, net['network']['id'], fixed_ips=fixed_ips)
                port = self.deserialize('json', port_res)
                self.assertIn('port', port)

                # try to create another one - should fail
                port_res = self._create_port(
                    self.fmt, net['network']['id'], fixed_ips=fixed_ips)
                port = self.deserialize('json', port_res)
                self.assertIn('NeutronError', port)
                self.assertIn('message', port['NeutronError'])
                self.assertTrue(('already allocated in subnet' in
                                 port['NeutronError']['message']))


class TestNsxvIpamPorts(test_plugin.TestPortsV2):
    """Run the nsxv plugin ports tests with the ipam driver"""
    def setUp(self):
        cfg.CONF.set_override(
            "ipam_driver",
            "vmware_nsx.services.ipam.nsx_v.driver.NsxvIpamDriver")
        super(TestNsxvIpamPorts, self).setUp()



# Copyright 2016 VMware, Inc.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from neutron_lib.api import attributes as lib_attrs
from oslo_config import cfg
from oslo_utils import uuidutils
import webob.exc


def _fixup_res_dict(context, attr_name, res_dict, check_allow_post=True):
    # This method is a replacement of _fixup_res_dict which is used in
    # neutron.plugin.common.utils. All this mock does is insert a uuid
    # for the id field if one is not found ONLY if running in api_replay_mode.
    if cfg.CONF.api_replay_mode and 'id' not in res_dict:
        res_dict['id'] = uuidutils.generate_uuid()
    attr_info = lib_attrs.RESOURCES[attr_name]
    attr_ops = lib_attrs.AttributeInfo(attr_info)
    try:
        attr_ops.populate_project_id(context, res_dict, True)
        lib_attrs.populate_project_info(attr_info)
        attr_ops.verify_attributes(res_dict)
    except webob.exc.HTTPBadRequest as e:
        # convert webob exception into ValueError as these functions are
        # for internal use. webob exception doesn't make sense.
        raise ValueError(e.detail)

    attr_ops.fill_post_defaults(res_dict, check_allow_post=check_allow_post)
    attr_ops.convert_values(res_dict)
    return res_dict



# Copyright 2016 VMware, Inc.
# All Rights Reserved
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import mock
import netaddr

from oslo_config import cfg
from oslo_utils import uuidutils

from vmware_nsx.tests.unit.nsx_v3 import test_plugin
from vmware_nsxlib.v3 import exceptions as nsx_lib_exc
from vmware_nsxlib.v3 import nsx_constants as error


class MockIPPools(object):

    def patch_nsxlib_ipam(self):
        self.nsx_pools = {}

        def _create_pool(*args, **kwargs):
            pool_id = uuidutils.generate_uuid()
            gateway_ip = None
            if kwargs.get('gateway_ip'):
                gateway_ip = str(kwargs['gateway_ip'])
            subnet = {"allocation_ranges": kwargs.get('allocation_ranges'),
                      "gateway_ip": gateway_ip,
                      "cidr": args[0]}
            pool = {'id': pool_id,
                    'subnets': [subnet]}
            self.nsx_pools[pool_id] = {'pool': pool, 'allocated': []}
            return {'id': pool_id}

        def _update_pool(pool_id, **kwargs):
            pool = self.nsx_pools[pool_id]['pool']
            subnet = pool['subnets'][0]
            if 'gateway_ip' in kwargs:
                if kwargs['gateway_ip']:
                    subnet["gateway_ip"] = str(kwargs['gateway_ip'])
                else:
                    del subnet["gateway_ip"]

            if 'allocation_ranges' in kwargs:
                if kwargs['allocation_ranges']:
                    subnet["allocation_ranges"] = kwargs['allocation_ranges']
                else:
                    del subnet["allocation_ranges"]

        def _delete_pool(pool_id):
            del self.nsx_pools[pool_id]

        def _get_pool(pool_id):
            return self.nsx_pools[pool_id]['pool']

        def _allocate_ip(*args, **kwargs):
            nsx_pool = self.nsx_pools[args[0]]
            if kwargs.get('ip_addr'):
                ip_addr = netaddr.IPAddress(kwargs['ip_addr'])
                # verify that this ip was not yet allocated
                if ip_addr in nsx_pool['allocated']:
                    raise nsx_lib_exc.ManagerError(
                        manager='dummy', operation='allocate',
                        details='IP already allocated',
                        error_code=error.ERR_CODE_IPAM_IP_ALLOCATED)
                # skip ip validation for this mock.
                nsx_pool['allocated'].append(ip_addr)
                return {'allocation_id': str(ip_addr)}
            # get an unused ip from the pool
            ranges = nsx_pool['pool']['subnets'][0]['allocation_ranges']
            for ip_range in ranges:
                r = netaddr.IPRange(ip_range['start'], ip_range['end'])
                for ip_addr in r:
                    if ip_addr not in nsx_pool['allocated']:
                        nsx_pool['allocated'].append(ip_addr)
                        return {'allocation_id': str(ip_addr)}
            # no IP was found
            raise nsx_lib_exc.ManagerError(
                manager='dummy', operation='allocate',
                details='All IPs in the pool are allocated',
                error_code=error.ERR_CODE_IPAM_POOL_EXHAUSTED)

        def _release_ip(*args, **kwargs):
            nsx_pool = self.nsx_pools[args[0]]
            ip_addr = netaddr.IPAddress(args[1])
            nsx_pool['allocated'].remove(ip_addr)

        mock.patch(
            "vmware_nsxlib.v3.resources.IpPool.get",
            side_effect=_get_pool).start()
        mock.patch(
            "vmware_nsxlib.v3.resources.IpPool.create",
            side_effect=_create_pool).start()
        mock.patch(
            "vmware_nsxlib.v3.resources.IpPool.update",
            side_effect=_update_pool).start()
        mock.patch(
            "vmware_nsxlib.v3.resources.IpPool.delete",
            side_effect=_delete_pool).start()
        mock.patch(
            "vmware_nsxlib.v3.resources.IpPool.allocate",
            side_effect=_allocate_ip).start()
        mock.patch(
            "vmware_nsxlib.v3.resources.IpPool.release",
            side_effect=_release_ip).start()


class TestNsxv3IpamSubnets(test_plugin.TestSubnetsV2, MockIPPools):
    """Run the nsxv3 plugin subnets tests with the ipam driver."""
    def setUp(self):
        cfg.CONF.set_override(
            "ipam_driver",
            "vmware_nsx.services.ipam.nsx_v3.driver.Nsxv3IpamDriver")
        super(TestNsxv3IpamSubnets, self).setUp()
        self.patch_nsxlib_ipam()

    def test_subnet_update_ipv4_and_ipv6_pd_slaac_subnets(self):
        self.skipTest('Update ipam subnet is not supported')

    def test_subnet_update_ipv4_and_ipv6_pd_v6stateless_subnets(self):
        self.skipTest('Update ipam subnet is not supported')


class TestNsxv3IpamPorts(test_plugin.TestPortsV2, MockIPPools):
    """Run the nsxv3 plugin ports tests with the ipam driver."""
    def setUp(self):
        cfg.CONF.set_override(
            "ipam_driver",
            "vmware_nsx.services.ipam.nsx_v3.driver.Nsxv3IpamDriver")
        super(TestNsxv3IpamPorts, self).setUp()
        self.patch_nsxlib_ipam()

    def test_create_port_invalid_fixed_ip_address_v6_pd_slaac(self):
        self.skipTest('Update ipam subnet is not supported')

    def test_update_port_invalid_subnet_v6_pd_slaac(self):
        self.skipTest('Update ipam subnet is not supported')

    def test_update_port_update_ip_address_only(self):
        self.skipTest('Update ipam subnet is not supported')

    def test_update_port_invalid_fixed_ip_address_v6_pd_slaac(self):
        self.skipTest('Update ipam subnet is not supported')



# Copyright (c) 2014 VMware, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from neutron.tests.unit.api.v2 import test_base
from neutron_lib import constants
from neutron_lib import exceptions

from vmware_nsx.nsxlib import mh as nsxlib
from vmware_nsx.nsxlib.mh import secgroup as secgrouplib
from vmware_nsx.tests.unit.nsxlib.mh import base

_uuid = test_base._uuid


class SecurityProfileTestCase(base.NsxlibTestCase):

    def test_create_and_get_security_profile(self):
        sec_prof = secgrouplib.create_security_profile(
            self.fake_cluster, _uuid(), 'pippo', {'name': 'test'})
        sec_prof_res = nsxlib.do_request(
            secgrouplib.HTTP_GET,
            nsxlib._build_uri_path('security-profile',
                                   resource_id=sec_prof['uuid']),
            cluster=self.fake_cluster)
        self.assertEqual(sec_prof['uuid'], sec_prof_res['uuid'])
        # Check for builtin rules
        self.assertEqual(len(sec_prof_res['logical_port_egress_rules']), 1)
        self.assertEqual(len(sec_prof_res['logical_port_ingress_rules']), 2)

    def test_create_and_get_default_security_profile(self):
        sec_prof = secgrouplib.create_security_profile(
            self.fake_cluster, _uuid(), 'pippo', {'name': 'default'})
        sec_prof_res = nsxlib.do_request(
            secgrouplib.HTTP_GET,
            nsxlib._build_uri_path('security-profile',
                                   resource_id=sec_prof['uuid']),
            cluster=self.fake_cluster)
        self.assertEqual(sec_prof['uuid'], sec_prof_res['uuid'])
        # Check for builtin rules
        self.assertEqual(len(sec_prof_res['logical_port_egress_rules']), 3)
        self.assertEqual(len(sec_prof_res['logical_port_ingress_rules']), 2)

    def test_update_security_profile_raise_not_found(self):
        self.assertRaises(exceptions.NotFound,
                          secgrouplib.update_security_profile,
                          self.fake_cluster,
                          _uuid(), 'tatore_magno(the great)')

    def test_update_security_profile(self):
        tenant_id = 'foo_tenant_uuid'
        secgroup_id = 'foo_secgroup_uuid'
        old_sec_prof = secgrouplib.create_security_profile(
            self.fake_cluster, tenant_id, secgroup_id,
            {'name': 'tatore_magno'})
        new_sec_prof = secgrouplib.update_security_profile(
            self.fake_cluster, old_sec_prof['uuid'], 'aaron_magno')
        self.assertEqual('aaron_magno', new_sec_prof['display_name'])

    def test_update_security_profile_rules(self):
        sec_prof = secgrouplib.create_security_profile(
            self.fake_cluster, _uuid(), 'pippo', {'name': 'test'})
        ingress_rule = {'ethertype': 'IPv4'}
        egress_rule = {'ethertype': 'IPv4', 'profile_uuid': 'xyz'}
        new_rules = {'logical_port_egress_rules': [egress_rule],
                     'logical_port_ingress_rules': [ingress_rule]}
        secgrouplib.update_security_group_rules(
            self.fake_cluster, sec_prof['uuid'], new_rules)
        sec_prof_res = nsxlib.do_request(
            nsxlib.HTTP_GET,
            nsxlib._build_uri_path('security-profile',
                                   resource_id=sec_prof['uuid']),
            cluster=self.fake_cluster)
        self.assertEqual(sec_prof['uuid'], sec_prof_res['uuid'])
        # Check for builtin rules
        self.assertEqual(len(sec_prof_res['logical_port_egress_rules']), 2)
        self.assertIn(egress_rule,
                      sec_prof_res['logical_port_egress_rules'])
        self.assertEqual(len(sec_prof_res['logical_port_ingress_rules']), 1)
        self.assertIn(ingress_rule,
                      sec_prof_res['logical_port_ingress_rules'])

    def test_update_security_profile_rules_noingress(self):
        sec_prof = secgrouplib.create_security_profile(
            self.fake_cluster, _uuid(), 'pippo', {'name': 'test'})
        hidden_ingress_rule = {'ethertype': 'IPv4',
                               'ip_prefix': '127.0.0.1/32'}
        egress_rule = {'ethertype': 'IPv4', 'profile_uuid': 'xyz'}
        new_rules = {'logical_port_egress_rules': [egress_rule],
                     'logical_port_ingress_rules': []}
        secgrouplib.update_security_group_rules(
            self.fake_cluster, sec_prof['uuid'], new_rules)
        sec_prof_res = nsxlib.do_request(
            nsxlib.HTTP_GET,
            nsxlib._build_uri_path('security-profile',
                                   resource_id=sec_prof['uuid']),
            cluster=self.fake_cluster)
        self.assertEqual(sec_prof['uuid'], sec_prof_res['uuid'])
        # Check for builtin rules
        self.assertEqual(len(sec_prof_res['logical_port_egress_rules']), 2)
        self.assertIn(egress_rule,
                      sec_prof_res['logical_port_egress_rules'])
        self.assertEqual(len(sec_prof_res['logical_port_ingress_rules']), 1)
        self.assertIn(hidden_ingress_rule,
                      sec_prof_res['logical_port_ingress_rules'])

    def test_update_security_profile_rules_summarize_port_range(self):
        sec_prof = secgrouplib.create_security_profile(
            self.fake_cluster, _uuid(), 'pippo', {'name': 'test'})
        ingress_rule = [{'ethertype': 'IPv4'}]
        egress_rules = [
            {'ethertype': 'IPv4', 'protocol': constants.PROTO_NUM_UDP,
             'port_range_min': 1, 'port_range_max': 65535}]
        new_rules = {'logical_port_egress_rules': egress_rules,
                     'logical_port_ingress_rules': [ingress_rule]}
        egress_rules_summarized = [{'ethertype': 'IPv4',
                                    'protocol': constants.PROTO_NUM_UDP}]
        secgrouplib.update_security_group_rules(
            self.fake_cluster, sec_prof['uuid'], new_rules)
        sec_prof_res = nsxlib.do_request(
            nsxlib.HTTP_GET,
            nsxlib._build_uri_path('security-profile',
                                   resource_id=sec_prof['uuid']),
            cluster=self.fake_cluster)
        self.assertEqual(sec_prof['uuid'], sec_prof_res['uuid'])

        # Check for builtin rules
        self.assertEqual(len(sec_prof_res['logical_port_ingress_rules']), 1)
        self.assertEqual(sec_prof_res['logical_port_egress_rules'],
                         egress_rules_summarized)
        self.assertIn(ingress_rule,
                      sec_prof_res['logical_port_ingress_rules'])

    def test_update_security_profile_rules_summarize_ip_prefix(self):
        sec_prof = secgrouplib.create_security_profile(
            self.fake_cluster, _uuid(), 'pippo', {'name': 'test'})
        ingress_rule = [{'ethertype': 'IPv4'}]
        egress_rules = [
            {'ethertype': 'IPv4', 'protocol': constants.PROTO_NUM_UDP,
             'ip_prefix': '0.0.0.0/0'},
            {'ethertype': 'IPv6', 'protocol': constants.PROTO_NUM_UDP,
             'ip_prefix': '::/0'}]
        new_rules = {'logical_port_egress_rules': egress_rules,
                     'logical_port_ingress_rules': [ingress_rule]}
        egress_rules_summarized = [
            {'ethertype': 'IPv4', 'protocol': constants.PROTO_NUM_UDP},
            {'ethertype': 'IPv6', 'protocol': constants.PROTO_NUM_UDP}]
        secgrouplib.update_security_group_rules(
            self.fake_cluster, sec_prof['uuid'], new_rules)
        sec_prof_res = nsxlib.do_request(
            nsxlib.HTTP_GET,
            nsxlib._build_uri_path('security-profile',
                                   resource_id=sec_prof['uuid']),
            cluster=self.fake_cluster)
        self.assertEqual(sec_prof['uuid'], sec_prof_res['uuid'])

        # Check for builtin rules
        self.assertEqual(len(sec_prof_res['logical_port_ingress_rules']), 1)
        self.assertEqual(sec_prof_res['logical_port_egress_rules'],
                         egress_rules_summarized)
        self.assertIn(ingress_rule,
                      sec_prof_res['logical_port_ingress_rules'])

    def test_update_security_profile_rules_summarize_subset(self):
        sec_prof = secgrouplib.create_security_profile(
            self.fake_cluster, _uuid(), 'pippo', {'name': 'test'})
        ingress_rule = [{'ethertype': 'IPv4'}]
        egress_rules = [
            {'ethertype': 'IPv4', 'protocol': constants.PROTO_NUM_UDP,
             'port_range_min': 1, 'port_range_max': 1,
             'remote_ip_prefix': '1.1.1.1/20'},
            {'ethertype': 'IPv4', 'protocol': constants.PROTO_NUM_UDP,
             'port_range_min': 2, 'port_range_max': 2,
             'profile_uuid': 'xyz'},
            {'ethertype': 'IPv4', 'protocol': constants.PROTO_NUM_UDP}]
        new_rules = {'logical_port_egress_rules': egress_rules,
                     'logical_port_ingress_rules': [ingress_rule]}
        egress_rules_summarized = [
            {'ethertype': 'IPv4', 'protocol': constants.PROTO_NUM_UDP}]
        secgrouplib.update_security_group_rules(
            self.fake_cluster, sec_prof['uuid'], new_rules)
        sec_prof_res = nsxlib.do_request(
            nsxlib.HTTP_GET,
            nsxlib._build_uri_path('security-profile',
                                   resource_id=sec_prof['uuid']),
            cluster=self.fake_cluster)
        self.assertEqual(sec_prof['uuid'], sec_prof_res['uuid'])

        # Check for builtin rules
        self.assertEqual(len(sec_prof_res['logical_port_ingress_rules']), 1)
        self.assertEqual(sec_prof_res['logical_port_egress_rules'],
                         egress_rules_summarized)
        self.assertIn(ingress_rule,
                      sec_prof_res['logical_port_ingress_rules'])

    def test_update_non_existing_securityprofile_raises(self):
        self.assertRaises(exceptions.NeutronException,
                          secgrouplib.update_security_group_rules,
                          self.fake_cluster, 'whatever',
                          {'logical_port_egress_rules': [],
                           'logical_port_ingress_rules': []})

    def test_delete_security_profile(self):
        sec_prof = secgrouplib.create_security_profile(
            self.fake_cluster, _uuid(), 'pippo', {'name': 'test'})
        secgrouplib.delete_security_profile(
            self.fake_cluster, sec_prof['uuid'])
        self.assertRaises(exceptions.NotFound,
                          nsxlib.do_request,
                          nsxlib.HTTP_GET,
                          nsxlib._build_uri_path(
                              'security-profile',
                              resource_id=sec_prof['uuid']),
                          cluster=self.fake_cluster)

    def test_delete_non_existing_securityprofile_raises(self):
        self.assertRaises(exceptions.NeutronException,
                          secgrouplib.delete_security_profile,
                          self.fake_cluster, 'whatever')



# Copyright 2017 VMware, Inc.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib.callbacks import registry
from oslo_config import cfg
from oslo_log import log as logging

from vmware_nsx.plugins.nsx_v.vshield.common import exceptions
from vmware_nsx.shell.admin.plugins.common import constants
from vmware_nsx.shell.admin.plugins.common import utils as admin_utils
from vmware_nsx.shell.admin.plugins.nsxv.resources import utils
from vmware_nsx.shell import resources as shell

LOG = logging.getLogger(__name__)


@admin_utils.output_header
def validate_configuration(resource, event, trigger, **kwargs):
    """Validate the nsxv configuration"""
    try:
        utils.NsxVPluginWrapper()
    except exceptions.Forbidden:
        LOG.error("Configuration validation failed: wrong VSM credentials "
                  "for %s", cfg.CONF.nsxv.manager_uri)
    except Exception as e:
        LOG.error("Configuration validation failed: %s", e)
    else:
        LOG.info("Configuration validation succeeded")


registry.subscribe(validate_configuration,
                   constants.CONFIG,
                   shell.Operations.VALIDATE.value)



# Copyright 2017 VMware, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""nsxv_lbv2_l7policy

Revision ID: 01a33f93f5fd
Revises: dd9fe5a3a526
Create Date: 2017-01-04 10:10:59.990122

"""

# revision identifiers, used by Alembic.
revision = '01a33f93f5fd'
down_revision = 'dd9fe5a3a526'
from alembic import op
import sqlalchemy as sa

from neutron.db import migration


# milestone identifier, used by neutron-db-manage
neutron_milestone = [migration.OCATA]


def upgrade():
    if migration.schema_has_table('lbaas_l7policies'):
        op.create_table(
            'nsxv_lbaas_l7policy_bindings',
            sa.Column('policy_id', sa.String(length=36), nullable=False),
            sa.Column('edge_id', sa.String(length=36), nullable=False),
            sa.Column('edge_app_rule_id',
                      sa.String(length=36), nullable=False),
            sa.Column('created_at', sa.DateTime(), nullable=True),
            sa.Column('updated_at', sa.DateTime(), nullable=True),
            sa.PrimaryKeyConstraint('policy_id'),
            sa.ForeignKeyConstraint(['policy_id'],
                                    ['lbaas_l7policies.id'],
                                    ondelete='CASCADE'))



# Copyright 2016 VMware, Inc.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import re

import netaddr

from neutron_lib.callbacks import registry
from neutron_lib import constants as const
from oslo_config import cfg
from oslo_log import log as logging

from vmware_nsx.common import config  # noqa
from vmware_nsx.common import utils as nsx_utils
from vmware_nsx.dhcp_meta import rpc as nsx_rpc
from vmware_nsx.plugins.nsx_v3 import availability_zones as nsx_az
from vmware_nsx.shell.admin.plugins.common import constants
from vmware_nsx.shell.admin.plugins.common import formatters
from vmware_nsx.shell.admin.plugins.common import utils as admin_utils
from vmware_nsx.shell.admin.plugins.nsxv3.resources import utils
import vmware_nsx.shell.resources as shell
from vmware_nsxlib.v3 import exceptions as nsx_exc
from vmware_nsxlib.v3 import nsx_constants

LOG = logging.getLogger(__name__)
neutron_client = utils.NeutronDbClient()


def _is_metadata_network(network):
    # If a Neutron network has only one subnet with 169.254.169.252/30 CIDR,
    # then it is an internal metadata network.
    if len(network['subnets']) == 1:
        subnet = neutron_client.get_subnet(network['subnets'][0])
        if subnet['cidr'] == nsx_rpc.METADATA_SUBNET_CIDR:
            return True
    return False


@admin_utils.output_header
def list_metadata_networks(resource, event, trigger, **kwargs):
    """List Metadata networks in Neutron."""
    if not cfg.CONF.nsx_v3.native_metadata_route:
        meta_networks = [network
                         for network in neutron_client.get_networks()
                         if _is_metadata_network(network)]
        LOG.info(formatters.output_formatter(constants.METADATA_PROXY,
                                             meta_networks,
                                             ['id', 'name', 'subnets']))
    else:
        nsxlib = utils.get_connected_nsxlib()
        tags = [{'scope': 'os-neutron-net-id'}]
        ports = nsxlib.search_by_tags(resource_type='LogicalPort', tags=tags)
        for port in ports['results']:
            if port['attachment']['attachment_type'] == 'METADATA_PROXY':
                net_id = None
                for tag in port['tags']:
                    if tag['scope'] == 'os-neutron-net-id':
                        net_id = tag['tag']
                        break
                uri = '/md-proxies/%s/%s/status' % (port['attachment']['id'],
                                                    port['logical_switch_id'])
                status = nsxlib.client.get(uri)
                LOG.info("Status for MD proxy on neutron network %s (logical "
                         "switch %s) is %s",
                         net_id,
                         port['logical_switch_id'],
                         status.get('proxy_status', 'Unknown'))


@admin_utils.output_header
def nsx_update_metadata_proxy(resource, event, trigger, **kwargs):
    """Update Metadata proxy for NSXv3 CrossHairs."""

    nsxlib = utils.get_connected_nsxlib()
    nsx_version = nsxlib.get_version()
    if not nsx_utils.is_nsx_version_1_1_0(nsx_version):
        LOG.error("This utility is not available for NSX version %s",
                  nsx_version)
        return

    metadata_proxy_uuid = None
    if kwargs.get('property'):
        properties = admin_utils.parse_multi_keyval_opt(kwargs['property'])
        metadata_proxy_uuid = properties.get('metadata_proxy_uuid')
    if not metadata_proxy_uuid:
        LOG.error("metadata_proxy_uuid is not defined")
        return

    cfg.CONF.set_override('dhcp_agent_notification', False)
    cfg.CONF.set_override('native_dhcp_metadata', True, 'nsx_v3')
    cfg.CONF.set_override('metadata_proxy', metadata_proxy_uuid, 'nsx_v3')

    with utils.NsxV3PluginWrapper() as plugin:
        # For each Neutron network, check if it is an internal metadata
        # network.
        # If yes, delete the network and associated router interface.
        # Otherwise, create a logical switch port with MD-Proxy attachment.
        for network in neutron_client.get_networks():
            if _is_metadata_network(network):
                # It is a metadata network, find the attached router,
                # remove the router interface and the network.
                filters = {'device_owner': const.ROUTER_INTERFACE_OWNERS,
                           'fixed_ips': {
                               'subnet_id': [network['subnets'][0]],
                               'ip_address': [nsx_rpc.METADATA_GATEWAY_IP]}}
                ports = neutron_client.get_ports(filters=filters)
                if not ports:
                    continue
                router_id = ports[0]['device_id']
                interface = {'subnet_id': network['subnets'][0]}
                plugin.remove_router_interface(router_id, interface)
                LOG.info("Removed metadata interface on router %s", router_id)
                plugin.delete_network(network['id'])
                LOG.info("Removed metadata network %s", network['id'])
            else:
                lswitch_id = neutron_client.net_id_to_lswitch_id(
                    network['id'])
                if not lswitch_id:
                    continue
                tags = nsxlib.build_v3_tags_payload(
                    network, resource_type='os-neutron-net-id',
                    project_name='admin')
                name = nsx_utils.get_name_and_uuid('%s-%s' % (
                    'mdproxy', network['name'] or 'network'), network['id'])
                # check if this logical port already exists
                existing_ports = nsxlib.logical_port.find_by_display_name(
                    name)
                if not existing_ports:
                    # create a new port with the md-proxy
                    nsxlib.logical_port.create(
                        lswitch_id, metadata_proxy_uuid, tags=tags, name=name,
                        attachment_type=nsx_constants.ATTACHMENT_MDPROXY)
                    LOG.info("Enabled native metadata proxy for network %s",
                             network['id'])
                else:
                    # update the MDproxy of this port
                    port = existing_ports[0]
                    nsxlib.logical_port.update(
                        port['id'], metadata_proxy_uuid,
                        attachment_type=nsx_constants.ATTACHMENT_MDPROXY)
                    LOG.info("Updated native metadata proxy for network %s",
                             network['id'])


@admin_utils.output_header
def nsx_update_metadata_proxy_server_ip(resource, event, trigger, **kwargs):
    """Update Metadata proxy server ip on the nsx."""
    nsxlib = utils.get_connected_nsxlib()
    nsx_version = nsxlib.get_version()
    if not nsx_utils.is_nsx_version_1_1_0(nsx_version):
        LOG.error("This utility is not available for NSX version %s",
                  nsx_version)
        return

    server_ip = None
    az_name = nsx_az.DEFAULT_NAME
    if kwargs.get('property'):
        properties = admin_utils.parse_multi_keyval_opt(kwargs['property'])
        server_ip = properties.get('server-ip')
        az_name = properties.get('availability-zone', az_name)
    if not server_ip or not netaddr.valid_ipv4(server_ip):
        LOG.error("Need to specify a valid server-ip parameter")
        return

    config.register_nsxv3_azs(cfg.CONF, cfg.CONF.nsx_v3.availability_zones)
    if (az_name != nsx_az.DEFAULT_NAME and
        az_name not in cfg.CONF.nsx_v3.availability_zones):
        LOG.error("Availability zone %s was not found in the configuration",
                  az_name)
        return

    az = nsx_az.NsxV3AvailabilityZones().get_availability_zone(az_name)
    az.translate_configured_names_to_uuids(nsxlib)

    if (not az.metadata_proxy or
        not cfg.CONF.nsx_v3.native_dhcp_metadata):
        LOG.error("Native DHCP metadata is not enabled in the configuration "
                  "of availability zone %s", az_name)
        return
    metadata_proxy_uuid = az._native_md_proxy_uuid

    try:
        mdproxy = nsxlib.native_md_proxy.get(metadata_proxy_uuid)
    except nsx_exc.ResourceNotFound:
        LOG.error("metadata proxy %s not found", metadata_proxy_uuid)
        return

    # update the IP in the URL
    url = mdproxy.get('metadata_server_url')
    url = re.sub(r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}', server_ip, url)
    LOG.info("Updating the URL of the metadata proxy server %(uuid)s to "
             "%(url)s", {'uuid': metadata_proxy_uuid, 'url': url})
    nsxlib.native_md_proxy.update(metadata_proxy_uuid, server_url=url)
    LOG.info("Done.")


registry.subscribe(list_metadata_networks,
                   constants.METADATA_PROXY,
                   shell.Operations.LIST.value)
registry.subscribe(nsx_update_metadata_proxy,
                   constants.METADATA_PROXY,
                   shell.Operations.NSX_UPDATE.value)
registry.subscribe(nsx_update_metadata_proxy_server_ip,
                   constants.METADATA_PROXY,
                   shell.Operations.NSX_UPDATE_IP.value)



# Copyright 2013 VMware, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import mock
from neutron.tests import base
from oslo_config import cfg
from oslo_utils import uuidutils
import six

from vmware_nsx.api_client import client
from vmware_nsx.api_client import version
from vmware_nsx.common import config  # noqa
from vmware_nsx.common import exceptions
from vmware_nsx.common import sync
from vmware_nsx import nsx_cluster
from vmware_nsx.nsxlib.mh import lsn as lsnlib
from vmware_nsx import plugin as mh_plugin
from vmware_nsx.tests import unit as vmware

BASE_CONF_PATH = vmware.get_fake_conf('neutron.conf.test')
NSX_INI_PATH = vmware.get_fake_conf('nsx.ini.basic.test')
NSX_INI_FULL_PATH = vmware.get_fake_conf('nsx.ini.full.test')
NSX_INI_AGENTLESS_PATH = vmware.get_fake_conf('nsx.ini.agentless.test')
NSX_INI_COMBINED_PATH = vmware.get_fake_conf('nsx.ini.combined.test')
NVP_INI_DEPR_PATH = vmware.get_fake_conf('nvp.ini.full.test')


class NSXClusterTest(base.BaseTestCase):

    cluster_opts = {'default_tz_uuid': uuidutils.generate_uuid(),
                    'default_l2_gw_service_uuid': uuidutils.generate_uuid(),
                    'default_l2_gw_service_uuid': uuidutils.generate_uuid(),
                    'nsx_user': 'foo',
                    'nsx_password': 'bar',
                    'http_timeout': 25,
                    'retries': 7,
                    'redirects': 23,
                    'nsx_default_interface_name': 'baz',
                    'nsx_controllers': ['1.1.1.1:443']}

    def test_create_cluster(self):
        cluster = nsx_cluster.NSXCluster(**self.cluster_opts)
        for (k, v) in six.iteritems(self.cluster_opts):
            self.assertEqual(v, getattr(cluster, k))

    def test_create_cluster_default_port(self):
        opts = self.cluster_opts.copy()
        opts['nsx_controllers'] = ['1.1.1.1']
        cluster = nsx_cluster.NSXCluster(**opts)
        for (k, v) in six.iteritems(self.cluster_opts):
            self.assertEqual(v, getattr(cluster, k))

    def test_create_cluster_missing_required_attribute_raises(self):
        opts = self.cluster_opts.copy()
        opts.pop('default_tz_uuid')
        self.assertRaises(exceptions.InvalidClusterConfiguration,
                          nsx_cluster.NSXCluster, **opts)


class ConfigurationTest(base.BaseTestCase):

    def setUp(self):
        super(ConfigurationTest, self).setUp()
        # Avoid runs of the synchronizer looping call
        patch_sync = mock.patch.object(sync, '_start_loopingcall')
        patch_sync.start()

    def _assert_required_options(self, cluster):
        self.assertEqual(cluster.nsx_controllers, ['fake_1:443', 'fake_2:443'])
        self.assertEqual(cluster.default_tz_uuid, 'fake_tz_uuid')
        self.assertEqual(cluster.nsx_user, 'foo')
        self.assertEqual(cluster.nsx_password, 'bar')

    def _assert_extra_options(self, cluster):
        self.assertEqual(13, cluster.http_timeout)
        self.assertEqual(12, cluster.redirects)
        self.assertEqual(11, cluster.retries)
        self.assertEqual('whatever', cluster.default_l2_gw_service_uuid)
        self.assertEqual('whatever', cluster.default_l3_gw_service_uuid)
        self.assertEqual('whatever', cluster.nsx_default_interface_name)

    def _get_mh_plugin(self):
        with mock.patch("neutron.common.rpc.create_connection"):
            plugin = mh_plugin.NsxPlugin()
        return plugin

    def test_load_plugin_with_full_options(self):
        self.config_parse(args=['--config-file', BASE_CONF_PATH,
                                '--config-file', NSX_INI_FULL_PATH])
        cfg.CONF.set_override('core_plugin', vmware.PLUGIN_NAME)
        plugin = self._get_mh_plugin()
        cluster = plugin.cluster
        self._assert_required_options(cluster)
        self._assert_extra_options(cluster)

    def test_load_plugin_with_required_options_only(self):
        self.config_parse(args=['--config-file', BASE_CONF_PATH,
                                '--config-file', NSX_INI_PATH])
        cfg.CONF.set_override('core_plugin', vmware.PLUGIN_NAME)
        plugin = self._get_mh_plugin()
        self._assert_required_options(plugin.cluster)

    def test_defaults(self):
        self.assertEqual(5000, cfg.CONF.NSX.max_lp_per_bridged_ls)
        self.assertEqual(256, cfg.CONF.NSX.max_lp_per_overlay_ls)
        self.assertEqual(10, cfg.CONF.NSX.concurrent_connections)
        self.assertEqual('access_network', cfg.CONF.NSX.metadata_mode)
        self.assertEqual('stt', cfg.CONF.NSX.default_transport_type)
        self.assertEqual('service', cfg.CONF.NSX.replication_mode)

        self.assertIsNone(cfg.CONF.default_tz_uuid)
        self.assertEqual('admin', cfg.CONF.nsx_user)
        self.assertEqual('admin', cfg.CONF.nsx_password)
        self.assertEqual(75, cfg.CONF.http_timeout)
        self.assertEqual(2, cfg.CONF.retries)
        self.assertEqual(2, cfg.CONF.redirects)
        self.assertEqual([], cfg.CONF.nsx_controllers)
        self.assertIsNone(cfg.CONF.default_l3_gw_service_uuid)
        self.assertIsNone(cfg.CONF.default_l2_gw_service_uuid)
        self.assertEqual('breth0', cfg.CONF.nsx_default_interface_name)
        self.assertEqual(900, cfg.CONF.conn_idle_timeout)

    def test_load_api_extensions(self):
        self.config_parse(args=['--config-file', BASE_CONF_PATH,
                                '--config-file', NSX_INI_FULL_PATH])
        cfg.CONF.set_override('core_plugin', vmware.PLUGIN_NAME)
        # Load the configuration, and initialize the plugin
        self._get_mh_plugin()
        self.assertIn('extensions', cfg.CONF.api_extensions_path)

    def test_agentless_extensions(self):
        self.config_parse(args=['--config-file', BASE_CONF_PATH,
                                '--config-file', NSX_INI_AGENTLESS_PATH])
        cfg.CONF.set_override('core_plugin', vmware.PLUGIN_NAME)
        self.assertEqual(config.AgentModes.AGENTLESS,
                         cfg.CONF.NSX.agent_mode)
        # The version returned from NSX matter as it has be exactly 4.1
        with mock.patch.object(client.NsxApiClient,
                               'get_version',
                               return_value=version.Version("4.1")):
            with mock.patch.object(lsnlib,
                                   'service_cluster_exists',
                                   return_value=True):
                plugin = self._get_mh_plugin()
                self.assertNotIn('agent',
                                 plugin.supported_extension_aliases)
                self.assertNotIn('dhcp_agent_scheduler',
                                 plugin.supported_extension_aliases)
                self.assertNotIn('lsn',
                                 plugin.supported_extension_aliases)

    def test_agentless_extensions_version_fail(self):
        self.config_parse(args=['--config-file', BASE_CONF_PATH,
                                '--config-file', NSX_INI_AGENTLESS_PATH])
        cfg.CONF.set_override('core_plugin', vmware.PLUGIN_NAME)
        self.assertEqual(config.AgentModes.AGENTLESS,
                         cfg.CONF.NSX.agent_mode)
        with mock.patch.object(client.NsxApiClient,
                               'get_version',
                               return_value=version.Version("3.2")):
            try:
                self._get_mh_plugin()
            except exceptions.NsxPluginException:
                # This is the correct result
                pass
            else:
                self.fail('Expected NsxPluginException exception')

    def test_agentless_extensions_unmet_deps_fail(self):
        self.config_parse(args=['--config-file', BASE_CONF_PATH,
                                '--config-file', NSX_INI_AGENTLESS_PATH])
        cfg.CONF.set_override('core_plugin', vmware.PLUGIN_NAME)
        self.assertEqual(config.AgentModes.AGENTLESS,
                         cfg.CONF.NSX.agent_mode)
        with mock.patch.object(client.NsxApiClient,
                               'get_version',
                               return_value=version.Version("3.2")):
            with mock.patch.object(lsnlib,
                                   'service_cluster_exists',
                                   return_value=False):
                try:
                    self._get_mh_plugin()
                except exceptions.NsxPluginException:
                    # This is the correct result
                    pass
                else:
                    self.fail('Expected NsxPluginException exception')

    def test_agent_extensions(self):
        self.config_parse(args=['--config-file', BASE_CONF_PATH,
                                '--config-file', NSX_INI_FULL_PATH])
        cfg.CONF.set_override('core_plugin', vmware.PLUGIN_NAME)
        self.assertEqual(config.AgentModes.AGENT,
                         cfg.CONF.NSX.agent_mode)
        plugin = self._get_mh_plugin()
        self.assertIn('agent',
                      plugin.supported_extension_aliases)
        self.assertIn('dhcp_agent_scheduler',
                      plugin.supported_extension_aliases)

    def test_combined_extensions(self):
        self.config_parse(args=['--config-file', BASE_CONF_PATH,
                                '--config-file', NSX_INI_COMBINED_PATH])
        cfg.CONF.set_override('core_plugin', vmware.PLUGIN_NAME)
        self.assertEqual(config.AgentModes.COMBINED,
                         cfg.CONF.NSX.agent_mode)
        with mock.patch.object(client.NsxApiClient,
                               'get_version',
                               return_value=version.Version("4.1")):
            with mock.patch.object(lsnlib,
                                   'service_cluster_exists',
                                   return_value=True):
                plugin = self._get_mh_plugin()
                self.assertIn('agent',
                              plugin.supported_extension_aliases)
                self.assertIn('dhcp_agent_scheduler',
                              plugin.supported_extension_aliases)
                self.assertIn('lsn',
                              plugin.supported_extension_aliases)


class OldNVPConfigurationTest(base.BaseTestCase):

    def setUp(self):
        super(OldNVPConfigurationTest, self).setUp()
        # Avoid runs of the synchronizer looping call
        patch_sync = mock.patch.object(sync, '_start_loopingcall')
        patch_sync.start()

    def _assert_required_options(self, cluster):
        self.assertEqual(cluster.nsx_controllers, ['fake_1:443', 'fake_2:443'])
        self.assertEqual(cluster.nsx_user, 'foo')
        self.assertEqual(cluster.nsx_password, 'bar')
        self.assertEqual(cluster.default_tz_uuid, 'fake_tz_uuid')

    def test_load_plugin_with_deprecated_options(self):
        self.config_parse(args=['--config-file', BASE_CONF_PATH,
                                '--config-file', NVP_INI_DEPR_PATH])
        cfg.CONF.set_override('core_plugin', vmware.PLUGIN_NAME)
        with mock.patch("neutron.common.rpc.create_connection"):
            plugin = mh_plugin.NsxPlugin()
            cluster = plugin.cluster
            # Verify old nvp_* params have been fully parsed
            self._assert_required_options(cluster)
            self.assertEqual(3, cluster.http_timeout)
            self.assertEqual(2, cluster.retries)
            self.assertEqual(2, cluster.redirects)



# Copyright 2014 VMware, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from neutron.tests.unit import testlib_api
from neutron_lib import context
from sqlalchemy import orm

from vmware_nsx.common import exceptions as p_exc
from vmware_nsx.db import lsn_db
from vmware_nsx.db import nsx_models


class LSNTestCase(testlib_api.SqlTestCase):

    def setUp(self):
        super(LSNTestCase, self).setUp()
        self.ctx = context.get_admin_context()
        self.net_id = 'foo_network_id'
        self.lsn_id = 'foo_lsn_id'
        self.lsn_port_id = 'foo_port_id'
        self.subnet_id = 'foo_subnet_id'
        self.mac_addr = 'aa:bb:cc:dd:ee:ff'

    def test_lsn_add(self):
        lsn_db.lsn_add(self.ctx, self.net_id, self.lsn_id)
        lsn = (self.ctx.session.query(nsx_models.Lsn).
               filter_by(lsn_id=self.lsn_id).one())
        self.assertEqual(self.lsn_id, lsn.lsn_id)

    def test_lsn_remove(self):
        lsn_db.lsn_add(self.ctx, self.net_id, self.lsn_id)
        lsn_db.lsn_remove(self.ctx, self.lsn_id)
        q = self.ctx.session.query(nsx_models.Lsn).filter_by(
            lsn_id=self.lsn_id)
        self.assertRaises(orm.exc.NoResultFound, q.one)

    def test_lsn_remove_for_network(self):
        lsn_db.lsn_add(self.ctx, self.net_id, self.lsn_id)
        lsn_db.lsn_remove_for_network(self.ctx, self.net_id)
        q = self.ctx.session.query(nsx_models.Lsn).filter_by(
            lsn_id=self.lsn_id)
        self.assertRaises(orm.exc.NoResultFound, q.one)

    def test_lsn_get_for_network(self):
        result = lsn_db.lsn_get_for_network(self.ctx, self.net_id,
                                            raise_on_err=False)
        self.assertIsNone(result)

    def test_lsn_get_for_network_raise_not_found(self):
        self.assertRaises(p_exc.LsnNotFound,
                          lsn_db.lsn_get_for_network,
                          self.ctx, self.net_id)

    def test_lsn_port_add(self):
        lsn_db.lsn_add(self.ctx, self.net_id, self.lsn_id)
        lsn_db.lsn_port_add_for_lsn(self.ctx, self.lsn_port_id,
                                    self.subnet_id, self.mac_addr, self.lsn_id)
        result = (self.ctx.session.query(nsx_models.LsnPort).
                  filter_by(lsn_port_id=self.lsn_port_id).one())
        self.assertEqual(self.lsn_port_id, result.lsn_port_id)

    def test_lsn_port_get_for_mac(self):
        lsn_db.lsn_add(self.ctx, self.net_id, self.lsn_id)
        lsn_db.lsn_port_add_for_lsn(self.ctx, self.lsn_port_id,
                                    self.subnet_id, self.mac_addr, self.lsn_id)
        result = lsn_db.lsn_port_get_for_mac(self.ctx, self.mac_addr)
        self.assertEqual(self.mac_addr, result.mac_addr)

    def test_lsn_port_get_for_mac_raise_not_found(self):
        self.assertRaises(p_exc.LsnPortNotFound,
                          lsn_db.lsn_port_get_for_mac,
                          self.ctx, self.mac_addr)

    def test_lsn_port_get_for_subnet(self):
        lsn_db.lsn_add(self.ctx, self.net_id, self.lsn_id)
        lsn_db.lsn_port_add_for_lsn(self.ctx, self.lsn_port_id,
                                    self.subnet_id, self.mac_addr, self.lsn_id)
        result = lsn_db.lsn_port_get_for_subnet(self.ctx, self.subnet_id)
        self.assertEqual(self.subnet_id, result.sub_id)

    def test_lsn_port_get_for_subnet_raise_not_found(self):
        self.assertRaises(p_exc.LsnPortNotFound,
                          lsn_db.lsn_port_get_for_subnet,
                          self.ctx, self.mac_addr)

    def test_lsn_port_remove(self):
        lsn_db.lsn_add(self.ctx, self.net_id, self.lsn_id)
        lsn_db.lsn_port_remove(self.ctx, self.lsn_port_id)
        q = (self.ctx.session.query(nsx_models.LsnPort).
             filter_by(lsn_port_id=self.lsn_port_id))
        self.assertRaises(orm.exc.NoResultFound, q.one)



# Copyright 2016 VMware, Inc.
#
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib import constants as n_consts
from neutron_lib.plugins import constants as plugin_const
from neutron_lib.plugins import directory
from neutron_lib.services.qos import constants as qos_consts

from oslo_config import cfg
from oslo_log import log as logging

LOG = logging.getLogger(__name__)


class NsxVQosBWLimits(object):
    # Data structure to hold the NSX-V representation
    # of the neutron QoS Bandwidth rule
    bandwidthEnabled = False
    averageBandwidth = 0
    peakBandwidth = 0
    burstSize = 0


class NsxVQosRule(object):

    def __init__(self, context=None, qos_policy_id=None):
        super(NsxVQosRule, self).__init__()

        self._qos_plugin = None

        # Data structure to hold the NSX-V representation
        # of the neutron QoS Bandwidth rule for both directions
        self.egress = NsxVQosBWLimits()
        self.ingress = NsxVQosBWLimits()

        # And data for the DSCP marking rule
        self.dscpMarkEnabled = False
        self.dscpMarkValue = 0

        if qos_policy_id is not None:
            self._init_from_policy_id(context, qos_policy_id)

    def _get_qos_plugin(self):
        if not self._qos_plugin:
            self._qos_plugin = directory.get_plugin(plugin_const.QOS)
        return self._qos_plugin

    # init the nsx_v qos data (outShapingPolicy) from a neutron qos policy
    def _init_from_policy_id(self, context, qos_policy_id):
        self.bandwidthEnabled = False
        self.dscpMarkEnabled = False

        # read the neutron policy restrictions
        if qos_policy_id is not None:
            plugin = self._get_qos_plugin()
            policy_obj = plugin.get_policy(context, qos_policy_id)
            if 'rules' in policy_obj and len(policy_obj['rules']) > 0:
                for rule_obj in policy_obj['rules']:
                    if (rule_obj['type'] ==
                        qos_consts.RULE_TYPE_BANDWIDTH_LIMIT):
                        # BW limit rule for one of the directions
                        if rule_obj['direction'] == n_consts.EGRESS_DIRECTION:
                            dir_obj = self.egress
                        else:
                            dir_obj = self.ingress
                        dir_obj.bandwidthEnabled = True
                        # averageBandwidth: kbps (neutron) -> bps (nsxv)
                        dir_obj.averageBandwidth = rule_obj['max_kbps'] * 1024
                        # peakBandwidth: a Multiplying on the average BW
                        # because the neutron qos configuration supports
                        # only 1 value
                        dir_obj.peakBandwidth = int(round(
                            dir_obj.averageBandwidth *
                            cfg.CONF.NSX.qos_peak_bw_multiplier))
                        # burstSize: kbps (neutron) -> Bytes (nsxv)
                        dir_obj.burstSize = rule_obj['max_burst_kbps'] * 128
                    if rule_obj['type'] == qos_consts.RULE_TYPE_DSCP_MARKING:
                        # DSCP marking rule
                        self.dscpMarkEnabled = True
                        self.dscpMarkValue = rule_obj['dscp_mark']

        return self



# Copyright 2017 VMware, Inc.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from neutron_lib import exceptions as n_exc
from oslo_log import helpers as log_helpers
from oslo_log import log as logging
from oslo_utils import excutils

from vmware_nsx._i18n import _
from vmware_nsx.db import db as nsx_db
from vmware_nsx.services.lbaas import base_mgr
from vmware_nsx.services.lbaas.nsx_v3 import lb_utils

LOG = logging.getLogger(__name__)


class EdgeL7RuleManager(base_mgr.Nsxv3LoadbalancerBaseManager):
    @log_helpers.log_method_call
    def __init__(self):
        super(EdgeL7RuleManager, self).__init__()

    def _update_l7rule_change(self, context, rule, delete=False):
        rule_client = self.core_plugin.nsxlib.load_balancer.rule
        binding = nsx_db.get_nsx_lbaas_l7policy_binding(context.session,
                                                        rule.policy.id)
        if not binding:
            self.lbv2_driver.l7rule.failed_completion(context, rule)
            msg = _('Cannot find nsx lbaas binding for policy '
                    '%(policy_id)s') % {'policy_id': rule.policy.id}
            raise n_exc.BadRequest(resource='lbaas-l7policy-update', msg=msg)

        lb_rule_id = binding['lb_rule_id']
        if delete:
            lb_utils.remove_rule_from_policy(rule)
        rule_body = lb_utils.convert_l7policy_to_lb_rule(context, rule.policy)
        try:
            rule_client.update(lb_rule_id, **rule_body)
        except Exception as e:
            with excutils.save_and_reraise_exception():
                self.lbv2_driver.l7rule.failed_completion(context, rule)
                LOG.error('Failed to update L7policy %(policy)s: '
                          '%(err)s', {'policy': rule.policy.id, 'err': e})

        self.lbv2_driver.l7rule.successful_completion(context, rule,
                                                      delete=delete)

    @log_helpers.log_method_call
    def create(self, context, rule):
        self._update_l7rule_change(context, rule)

    @log_helpers.log_method_call
    def update(self, context, old_rule, new_rule):
        self._update_l7rule_change(context, new_rule)

    @log_helpers.log_method_call
    def delete(self, context, rule):
        self._update_l7rule_change(context, rule, delete=True)



# Copyright 2013 VMware, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from neutron.db import models_v2
from neutron.tests.unit import testlib_api
from neutron_lib import context
from oslo_db import exception as d_exc

from vmware_nsx.db import db as nsx_db


class NsxDBTestCase(testlib_api.SqlTestCase):

    def setUp(self):
        super(NsxDBTestCase, self).setUp()
        self.ctx = context.get_admin_context()

    def _setup_neutron_network_and_port(self, network_id, port_id):
        with self.ctx.session.begin(subtransactions=True):
            self.ctx.session.add(models_v2.Network(id=network_id))
            port = models_v2.Port(id=port_id,
                                  network_id=network_id,
                                  mac_address='foo_mac_address',
                                  admin_state_up=True,
                                  status='ACTIVE',
                                  device_id='',
                                  device_owner='')
            self.ctx.session.add(port)

    def test_add_neutron_nsx_port_mapping_raise_integrity_constraint(self):
        neutron_port_id = 'foo_neutron_port_id'
        nsx_port_id = 'foo_nsx_port_id'
        nsx_switch_id = 'foo_nsx_switch_id'
        self.assertRaises(d_exc.DBError,
                          nsx_db.add_neutron_nsx_port_mapping,
                          self.ctx.session, neutron_port_id,
                          nsx_switch_id, nsx_port_id)



# Copyright 2017 VMware, Inc.
# All Rights Reserved
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_config import cfg
from oslo_utils import uuidutils

from neutron.tests import base

from vmware_nsx.common import config
from vmware_nsx.common import exceptions as nsx_exc
from vmware_nsx.plugins.nsx_v3 import availability_zones as nsx_az


class Nsxv3AvailabilityZonesTestCase(base.BaseTestCase):

    def setUp(self):
        super(Nsxv3AvailabilityZonesTestCase, self).setUp()
        self.az_name = "zone1"
        self.group_name = "az:%s" % self.az_name
        config.register_nsxv3_azs(cfg.CONF, [self.az_name])
        self.global_md_proxy = uuidutils.generate_uuid()
        cfg.CONF.set_override(
            "metadata_proxy", self.global_md_proxy, group="nsx_v3")
        self.global_dhcp_profile = uuidutils.generate_uuid()
        cfg.CONF.set_override(
            "dhcp_profile", self.global_dhcp_profile, group="nsx_v3")
        cfg.CONF.set_override(
            "native_metadata_route", "1.1.1.1", group="nsx_v3")
        cfg.CONF.set_override("dns_domain", "xxx.com", group="nsx_v3")
        cfg.CONF.set_override("nameservers", ["10.1.1.1"], group="nsx_v3")
        cfg.CONF.set_override("switching_profiles", ["uuid1"], group="nsx_v3")
        cfg.CONF.set_override("dhcp_relay_service", "service1", group="nsx_v3")

    def _config_az(self,
                   metadata_proxy="metadata_proxy1",
                   dhcp_profile="dhcp_profile1",
                   native_metadata_route="2.2.2.2",
                   dns_domain="aaa.com",
                   nameservers=["20.1.1.1"],
                   default_overlay_tz='otz',
                   default_vlan_tz='vtz',
                   switching_profiles=["uuid2"],
                   dhcp_relay_service="service2"):
        if metadata_proxy is not None:
            cfg.CONF.set_override("metadata_proxy", metadata_proxy,
                                  group=self.group_name)
        if dhcp_profile is not None:
            cfg.CONF.set_override("dhcp_profile", dhcp_profile,
                                  group=self.group_name)
        if native_metadata_route is not None:
            cfg.CONF.set_override("native_metadata_route",
                                  native_metadata_route,
                                  group=self.group_name)
        if dns_domain is not None:
            cfg.CONF.set_override("dns_domain", dns_domain,
                                  group=self.group_name)
        if nameservers is not None:
            cfg.CONF.set_override("nameservers", nameservers,
                                  group=self.group_name)
        if default_overlay_tz is not None:
            cfg.CONF.set_override("default_overlay_tz", default_overlay_tz,
                                  group=self.group_name)
        if default_vlan_tz is not None:
            cfg.CONF.set_override("default_vlan_tz", default_vlan_tz,
                                  group=self.group_name)
        if switching_profiles is not None:
            cfg.CONF.set_override("switching_profiles", switching_profiles,
                                  group=self.group_name)
        if dhcp_relay_service is not None:
            cfg.CONF.set_override("dhcp_relay_service", dhcp_relay_service,
                                  group=self.group_name)

    def test_simple_availability_zone(self):
        self._config_az()
        az = nsx_az.NsxV3AvailabilityZone(self.az_name)
        self.assertEqual(self.az_name, az.name)
        self.assertEqual("metadata_proxy1", az.metadata_proxy)
        self.assertEqual("dhcp_profile1", az.dhcp_profile)
        self.assertEqual("2.2.2.2", az.native_metadata_route)
        self.assertEqual("aaa.com", az.dns_domain)
        self.assertEqual(["20.1.1.1"], az.nameservers)
        self.assertEqual("otz", az.default_overlay_tz)
        self.assertEqual("vtz", az.default_vlan_tz)
        self.assertEqual(["uuid2"], az.switching_profiles)
        self.assertEqual("service2", az.dhcp_relay_service)

    def test_missing_group_section(self):
        self.assertRaises(
            nsx_exc.NsxInvalidConfiguration,
            nsx_az.NsxV3AvailabilityZone,
            "doesnt_exist")

    def test_availability_zone_missing_metadata_proxy(self):
        # Mandatory parameter
        self._config_az(metadata_proxy=None)
        self.assertRaises(
            nsx_exc.NsxInvalidConfiguration,
            nsx_az.NsxV3AvailabilityZone,
            self.az_name)

    def test_availability_zone_missing_dhcp_profile(self):
        # Mandatory parameter
        self._config_az(dhcp_profile=None)
        self.assertRaises(
            nsx_exc.NsxInvalidConfiguration,
            nsx_az.NsxV3AvailabilityZone,
            self.az_name)

    def test_availability_zone_missing_md_route(self):
        self._config_az(native_metadata_route=None)
        az = nsx_az.NsxV3AvailabilityZone(self.az_name)
        self.assertEqual("1.1.1.1", az.native_metadata_route)

    def test_availability_zone_missing_dns_domain(self):
        self._config_az(dns_domain=None)
        az = nsx_az.NsxV3AvailabilityZone(self.az_name)
        self.assertEqual("xxx.com", az.dns_domain)

    def test_availability_zone_missing_nameservers(self):
        self._config_az(nameservers=None)
        az = nsx_az.NsxV3AvailabilityZone(self.az_name)
        self.assertEqual(["10.1.1.1"], az.nameservers)

    def test_availability_zone_missing_profiles(self):
        self._config_az(switching_profiles=None)
        az = nsx_az.NsxV3AvailabilityZone(self.az_name)
        self.assertEqual(["uuid1"], az.switching_profiles)



# Copyright (c) 2013 OpenStack Foundation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock
from neutron.extensions import agent
from neutron.tests.unit.db import test_db_base_plugin_v2 as test_db_plugin
from neutron_lib import context
from oslo_config import cfg

from vmware_nsx.api_client import version
from vmware_nsx.common import sync
from vmware_nsx.tests import unit as vmware
from vmware_nsx.tests.unit.nsx_mh.apiclient import fake
from vmware_nsx.tests.unit import test_utils


class MacLearningExtensionManager(object):

    def get_resources(self):
        return agent.Agent.get_resources()

    def get_actions(self):
        return []

    def get_request_extensions(self):
        return []


class MacLearningDBTestCase(test_db_plugin.NeutronDbPluginV2TestCase):
    fmt = 'json'

    def setUp(self):
        test_utils.override_nsx_ini_full_test()
        cfg.CONF.set_override('api_extensions_path', vmware.NSXEXT_PATH)
        ext_mgr = MacLearningExtensionManager()
        # mock api client
        self.fc = fake.FakeClient(vmware.STUBS_PATH)
        self.mock_nsx = mock.patch(vmware.NSXAPI_NAME, autospec=True)
        instance = self.mock_nsx.start()
        # Avoid runs of the synchronizer looping call
        patch_sync = mock.patch.object(sync, '_start_loopingcall')
        patch_sync.start()

        # Emulate tests against NSX 2.x
        instance.return_value.get_version.return_value = version.Version("3.0")
        instance.return_value.request.side_effect = self.fc.fake_request
        cfg.CONF.set_override('metadata_mode', None, 'NSX')
        self.addCleanup(self.fc.reset_all)
        super(MacLearningDBTestCase, self).setUp(plugin=vmware.PLUGIN_NAME,
                                                 ext_mgr=ext_mgr)
        self.adminContext = context.get_admin_context()

    def test_create_with_mac_learning(self):
        with self.port(arg_list=('mac_learning_enabled',),
                       mac_learning_enabled=True) as port:
            # Validate create operation response
            self.assertEqual(True, port['port']['mac_learning_enabled'])
            # Verify that db operation successfully set mac learning state
            req = self.new_show_request('ports', port['port']['id'], self.fmt)
            sport = self.deserialize(self.fmt, req.get_response(self.api))
            self.assertEqual(True, sport['port']['mac_learning_enabled'])

    def test_create_and_show_port_without_mac_learning(self):
        with self.port() as port:
            req = self.new_show_request('ports', port['port']['id'], self.fmt)
            sport = self.deserialize(self.fmt, req.get_response(self.api))
            self.assertNotIn('mac_learning_enabled', sport['port'])

    def test_update_port_with_mac_learning(self):
        with self.port(arg_list=('mac_learning_enabled',),
                       mac_learning_enabled=False) as port:
            data = {'port': {'mac_learning_enabled': True}}
            req = self.new_update_request('ports', data, port['port']['id'])
            res = self.deserialize(self.fmt, req.get_response(self.api))
            self.assertEqual(True, res['port']['mac_learning_enabled'])

    def test_update_preexisting_port_with_mac_learning(self):
        with self.port() as port:
            req = self.new_show_request('ports', port['port']['id'], self.fmt)
            sport = self.deserialize(self.fmt, req.get_response(self.api))
            self.assertNotIn('mac_learning_enabled', sport['port'])
            data = {'port': {'mac_learning_enabled': True}}
            req = self.new_update_request('ports', data, port['port']['id'])
            # Validate update operation response
            res = self.deserialize(self.fmt, req.get_response(self.api))
            self.assertEqual(True, res['port']['mac_learning_enabled'])
            # Verify that db operation successfully updated mac learning state
            req = self.new_show_request('ports', port['port']['id'], self.fmt)
            sport = self.deserialize(self.fmt, req.get_response(self.api))
            self.assertEqual(True, sport['port']['mac_learning_enabled'])

    def test_list_ports(self):
        # for this test we need to enable overlapping ips
        cfg.CONF.set_default('allow_overlapping_ips', True)
        no_mac_learning_p = (lambda:
                             self.port(arg_list=('mac_learning_enabled',),
                                       mac_learning_enabled=True))

        with no_mac_learning_p(), no_mac_learning_p(), no_mac_learning_p():
            for port in self._list('ports')['ports']:
                self.assertEqual(True, port['mac_learning_enabled'])

    def test_show_port(self):
        with self.port(arg_list=('mac_learning_enabled',),
                       mac_learning_enabled=True) as p:
            port_res = self._show('ports', p['port']['id'])['port']
            self.assertEqual(True, port_res['mac_learning_enabled'])



# Copyright 2012 VMware, Inc.
#
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from oslo_log import log as logging

LOG = logging.getLogger(__name__)


def find_version(headers):
    """Retrieve NSX controller version from response headers."""
    for (header_name, header_value) in (headers or ()):
        try:
            if header_name == 'server':
                return Version(header_value.split('/')[1])
        except IndexError:
            LOG.warning("Unable to fetch NSX version from response "
                        "headers :%s", headers)


class Version(object):
    """Abstracts NSX version by exposing major and minor."""

    def __init__(self, version):
        self.full_version = version.split('.')
        self.major = int(self.full_version[0])
        self.minor = int(self.full_version[1])

    def __str__(self):
        return '.'.join(self.full_version)



# Copyright 2015 VMware, Inc.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log import log as logging

from vmware_nsx.shell.admin.plugins.common import constants
from vmware_nsx.shell.admin.plugins.common import formatters
from vmware_nsx.shell.admin.plugins.common import utils as admin_utils
from vmware_nsx.shell.admin.plugins.nsxv3.resources import utils
from vmware_nsxlib.v3 import nsx_constants as consts

LOG = logging.getLogger(__name__)


@admin_utils.list_handler(constants.LB_SERVICES)
@admin_utils.output_header
def nsx_list_lb_services(resource, event, trigger, **kwargs):
    """List LB services on NSX backend"""

    nsxlib = utils.get_connected_nsxlib()
    if not nsxlib.feature_supported(consts.FEATURE_LOAD_BALANCER):
        LOG.error("This utility is not available for NSX version %s",
                  nsxlib.get_version())
        return

    lb_services = nsxlib.load_balancer.service.list()
    LOG.info(formatters.output_formatter(
        constants.LB_SERVICES, [lb_services['results']],
        ['display_name', 'id', 'virtual_server_ids', 'attachment']))
    return bool(lb_services)


@admin_utils.list_handler(constants.LB_VIRTUAL_SERVERS)
@admin_utils.output_header
def nsx_list_lb_virtual_servers(resource, event, trigger, **kwargs):
    """List LB virtual servers on NSX backend"""

    nsxlib = utils.get_connected_nsxlib()
    if not nsxlib.feature_supported(consts.FEATURE_LOAD_BALANCER):
        LOG.error("This utility is not available for NSX version %s",
                  nsxlib.get_version())
        return

    lb_virtual_servers = nsxlib.load_balancer.virtual_server.list()
    LOG.info(formatters.output_formatter(
        constants.LB_VIRTUAL_SERVERS, [lb_virtual_servers['results']],
        ['display_name', 'id', 'ip_address', 'pool_id']))
    return bool(lb_virtual_servers)


@admin_utils.list_handler(constants.LB_POOLS)
@admin_utils.output_header
def nsx_list_lb_pools(resource, event, trigger, **kwargs):

    nsxlib = utils.get_connected_nsxlib()
    if not nsxlib.feature_supported(consts.FEATURE_LOAD_BALANCER):
        LOG.error("This utility is not available for NSX version %s",
                  nsxlib.get_version())
        return

    lb_pools = nsxlib.load_balancer.pool.list()
    LOG.info(formatters.output_formatter(
        constants.LB_POOLS, [lb_pools['results']],
        ['display_name', 'id', 'active_monitor_ids', 'members']))
    return bool(lb_pools)


@admin_utils.list_handler(constants.LB_MONITORS)
@admin_utils.output_header
def nsx_list_lb_monitors(resource, event, trigger, **kwargs):

    nsxlib = utils.get_connected_nsxlib()
    if not nsxlib.feature_supported(consts.FEATURE_LOAD_BALANCER):
        LOG.error("This utility is not available for NSX version %s",
                  nsxlib.get_version())
        return

    lb_monitors = nsxlib.load_balancer.monitor.list()
    LOG.info(formatters.output_formatter(
        constants.LB_MONITORS, [lb_monitors['results']],
        ['display_name', 'id', 'resource_type']))
    return bool(lb_monitors)



# Copyright 2015 VMware, Inc.
# All Rights Reserved
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import mock

from neutron.tests import base

from networking_l2gw.db.l2gateway import l2gateway_db
from neutron_lib import context
from neutron_lib import exceptions as n_exc
from vmware_nsx.common import exceptions as nsx_exc
from vmware_nsx.db import nsxv_db
from vmware_nsx.dvs import dvs_utils
from vmware_nsx.services.l2gateway.nsx_v import driver as nsx_v_driver
from vmware_nsx.tests.unit.nsx_v import test_plugin

CORE_PLUGIN = "vmware_nsx.plugins.nsx_v.plugin.NsxVPluginV2"


class TestL2gatewayDriver(base.BaseTestCase):

    def setUp(self):
        super(TestL2gatewayDriver, self).setUp()
        self.context = context.get_admin_context()
        self.plugin = nsx_v_driver.NsxvL2GatewayDriver(mock.MagicMock())

    def test_validate_device_with_multi_devices(self):
        fake_l2gw_dict = {"l2_gateway":
                          {"tenant_id": "fake__tenant_id",
                           "name": "fake_l2gw",
                           "devices": [{"interfaces":
                                        [{"name": "fake_inter"}],
                                        "device_name": "fake_dev"},
                                       {"interfaces":
                                        [{"name": "fake_inter_1"}],
                                        "device_name": "fake_dev_1"}]}}
        with mock.patch.object(l2gateway_db.L2GatewayMixin, '_admin_check'):
            self.assertRaises(n_exc.InvalidInput,
                              self.plugin.create_l2_gateway,
                              self.context, fake_l2gw_dict)

    def test_validate_interface_with_multi_interfaces(self):
        fake_l2gw_dict = {"l2_gateway":
                          {"tenant_id": "fake_tenant_id",
                           "name": "fake_l2gw",
                           "devices": [{"interfaces":
                                        [{"name": "fake_inter_1"},
                                         {"name": "fake_inter_2"}],
                                        "device_name": "fake_dev"}]}}
        with mock.patch.object(l2gateway_db.L2GatewayMixin, '_admin_check'):
            self.assertRaises(n_exc.InvalidInput,
                              self.plugin.create_l2_gateway,
                              self.context, fake_l2gw_dict)

    @mock.patch('vmware_nsx.services.l2gateway.'
                'nsx_v.driver.NsxvL2GatewayDriver._nsxv')
    def test_validate_interface_with_invalid_interfaces(self, _nsxv):
        fake_interfaces = [{"name": "fake_inter"}]
        _nsxv.vcns.validate_network.return_value = False
        self.assertRaises(n_exc.InvalidInput,
                          self.plugin._validate_interface_list,
                          self.context,
                          fake_interfaces)

    @mock.patch('vmware_nsx.services.l2gateway.'
                'nsx_v.driver.NsxvL2GatewayDriver._edge_manager')
    def test_create_gw_edge_failure(self, edge_manager):
        with mock.patch.object(nsxv_db,
                               'get_nsxv_router_binding',
                               return_value=None):
            self.assertRaises(nsx_exc.NsxL2GWDeviceNotFound,
                             self.plugin._create_l2_gateway_edge,
                             self.context)

    @mock.patch('networking_l2gw.db.l2gateway.l2gateway_db.'
                'L2GatewayMixin._admin_check')
    @mock.patch('vmware_nsx.services.l2gateway.'
                'nsx_v.driver.NsxvL2GatewayDriver._validate_device_list')
    @mock.patch('vmware_nsx.services.l2gateway.'
                'nsx_v.driver.NsxvL2GatewayDriver._validate_interface_list')
    @mock.patch('vmware_nsx.services.l2gateway.'
                'nsx_v.driver.NsxvL2GatewayDriver._create_l2_gateway_edge')
    @mock.patch('networking_l2gw.db.l2gateway.l2gateway_db.'
                'L2GatewayMixin.create_l2_gateway')
    def test_create_l2_gateway_failure(self, create_l2gw, _create_l2gw_edge,
                               val_inter, val_dev, _admin_check):
        fake_l2gw_dict = {"l2_gateway":
                          {"tenant_id": "fake_teannt_id",
                           "name": "fake_l2gw",
                           "devices": [{"interfaces":
                                        [{"name": "fake_inter"}],
                                        "device_name": "fake_dev"}]}}
        _create_l2gw_edge.side_effect = nsx_exc.NsxL2GWDeviceNotFound
        self.assertRaises(nsx_exc.NsxL2GWDeviceNotFound,
                         self.plugin.create_l2_gateway,
                         self.context, fake_l2gw_dict)

    @mock.patch('networking_l2gw.db.l2gateway.l2gateway_db.'
                'L2GatewayMixin._admin_check')
    @mock.patch('vmware_nsx.services.l2gateway.'
                'nsx_v.driver.NsxvL2GatewayDriver._validate_device_list')
    @mock.patch('vmware_nsx.services.l2gateway.'
                'nsx_v.driver.NsxvL2GatewayDriver._validate_interface_list')
    @mock.patch('vmware_nsx.services.l2gateway.'
                'nsx_v.driver.NsxvL2GatewayDriver._create_l2_gateway_edge')
    @mock.patch('vmware_nsx.services.l2gateway.'
                'nsx_v.driver.NsxvL2GatewayDriver._edge_manager')
    def test_create_l2_gateway(self, edge_manager, _create_l2gw_edge,
                               val_inter, val_dev, _admin_check):
        fake_l2gw_dict = {"l2_gateway":
                          {"tenant_id": "fake_teannt_id",
                           "name": "fake_l2gw",
                           "devices": [{"interfaces":
                                        [{"name": "fake_inter"}],
                                        "device_name": "fake_dev"}]}}
        fake_devices = [{"interfaces": [{"name": "fake_inter"}],
                         "device_name": "fake_dev"}]
        fake_interfaces = [{"name": "fake_inter"}]
        _create_l2gw_edge.return_value = 'fake_dev'
        self.plugin.create_l2_gateway(self.context, fake_l2gw_dict)
        _admin_check.assert_called_with(self.context, 'CREATE')
        val_dev.assert_called_with(fake_devices)
        val_inter.assert_called_with(self.context, fake_interfaces)

    @mock.patch('networking_l2gw.db.l2gateway.l2gateway_db.'
                'L2GatewayMixin._admin_check')
    @mock.patch('networking_l2gw.db.l2gateway.l2gateway_db.'
                'L2GatewayMixin.get_l2_gateway_connection')
    @mock.patch('vmware_nsx.services.l2gateway.'
                'nsx_v.driver.NsxvL2GatewayDriver._get_device')
    @mock.patch('vmware_nsx.services.l2gateway.'
                'nsx_v.driver.NsxvL2GatewayDriver._nsxv')
    def test_delete_l2_gateway_connection(self, nsxv, get_devices,
                                          get_conn, admin_check):
        fake_conn_dict = {'l2_gateway_id': 'fake_l2gw_id'}
        fake_device_dict = {'id': 'fake_dev_id',
                            'device_name': 'fake_dev_name'}
        get_conn.return_value = fake_conn_dict
        get_devices.return_value = fake_device_dict
        self.plugin.delete_l2_gateway_connection(self.context, fake_conn_dict)
        admin_check.assert_called_with(self.context, 'DELETE')
        get_conn.assert_called_with(self.context, fake_conn_dict)
        get_devices.assert_called_with(self.context, 'fake_l2gw_id')
        self.plugin._nsxv().del_bridge.asert_called_with('fake_dev_name')

    @mock.patch('networking_l2gw.db.l2gateway.l2gateway_db.'
                'L2GatewayMixin._admin_check')
    @mock.patch('vmware_nsx.services.l2gateway.'
                'nsx_v.driver.NsxvL2GatewayDriver._get_device')
    @mock.patch('vmware_nsx.db.'
                'nsxv_db.get_nsxv_router_binding_by_edge')
    @mock.patch('vmware_nsx.services.l2gateway.'
                'nsx_v.driver.NsxvL2GatewayDriver._edge_manager')
    def test_delete_l2_gateway(self, edge_manager, get_nsxv_router,
                               get_devices, admin_check):
        fake_device_dict = {"id": "fake_dev_id",
                            "device_name": "fake_edge_name",
                            "l2_gateway_id": "fake_l2gw_id"}
        fake_rtr_binding = {"router_id": 'fake_router_id'}
        get_devices.return_value = fake_device_dict
        get_nsxv_router.return_value = fake_rtr_binding
        self.plugin.delete_l2_gateway(self.context, 'fake_l2gw_id')
        admin_check.assert_called_with(self.context, 'DELETE')
        get_devices.assert_called_with(self.context, 'fake_l2gw_id')
        get_nsxv_router.assert_called_with(self.context.session,
                                           "fake_edge_name")


class TestL2GatewayDriverRouter(test_plugin.NsxVPluginV2TestCase):

    @mock.patch.object(dvs_utils, 'dvs_create_session')
    def setUp(self, *mocks):
        # init the nsxv plugin, edge manager and fake vcns
        super(TestL2GatewayDriverRouter, self).setUp(plugin=CORE_PLUGIN,
                                                     ext_mgr=None)
        self.context = context.get_admin_context()
        # init the L2 gateway driver
        self.driver = nsx_v_driver.NsxvL2GatewayDriver(mock.MagicMock())

    @mock.patch('vmware_nsx.services.l2gateway.'
                'nsx_v.driver.NsxvL2GatewayDriver._validate_device_list')
    @mock.patch('vmware_nsx.services.l2gateway.'
                'nsx_v.driver.NsxvL2GatewayDriver._validate_interface_list')
    def test_create_l2_gateway_router(self, val_inter, val_dev):
        # Verify that creating the router doesn't fail
        fake_l2gw_dict = {"l2_gateway":
                          {"tenant_id": "fake_teannt_id",
                           "name": "fake_l2gw",
                           "devices": [{"interfaces":
                                        [{"name": "fake_inter"}],
                                        "device_name": "fake_dev"}]}}
        self.driver.create_l2_gateway(self.context, fake_l2gw_dict)

    def test_create_l2_gateway_router_edge(self):
        # Verify that the router edge is really created
        edge_id = self.driver._create_l2_gateway_edge(self.context)
        self.assertEqual('edge-1', edge_id)



# Copyright (c) 2017 OpenStack Foundation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock

from oslo_config import cfg

from neutron_lib import context

from vmware_nsx.db import nsxv_db
from vmware_nsx.db import nsxv_models
from vmware_nsx.plugins.nsx_v.vshield.common import (
    constants as vcns_const)
from vmware_nsx.plugins.nsx_v.vshield import edge_utils
from vmware_nsx.tests.unit.nsx_v import test_plugin


PLUGIN_NAME = 'vmware_nsx.plugin.NsxVPlugin'


# Run all relevant plugin tests when the metadata proxy is enabled.
# Those tests does not specifically test the md_proxy. just verify that
# nothing gets broken.
class NsxVPluginWithMdV2TestCase(test_plugin.NsxVPluginV2TestCase):

    def setUp(self, plugin=PLUGIN_NAME,
              ext_mgr=None,
              service_plugins=None):
        # Add the metadata configuration
        cfg.CONF.set_override('mgt_net_moid', 'net-1', group="nsxv")
        cfg.CONF.set_override('mgt_net_proxy_ips', ['2.2.2.2'], group="nsxv")
        cfg.CONF.set_override('mgt_net_proxy_netmask', '255.255.255.0',
                              group="nsxv")
        cfg.CONF.set_override('mgt_net_default_gateway', '1.1.1.1',
                              group="nsxv")
        cfg.CONF.set_override('nova_metadata_ips', ['3.3.3.3'], group="nsxv")

        # Add some mocks required for the md code
        mock_alloc_vnic = mock.patch.object(nsxv_db, 'allocate_edge_vnic')
        mock_alloc_vnic_inst = mock_alloc_vnic.start()
        mock_alloc_vnic_inst.return_value = nsxv_models.NsxvEdgeVnicBinding
        mock.patch.object(edge_utils, "update_internal_interface").start()

        super(NsxVPluginWithMdV2TestCase, self).setUp(
            plugin=plugin, ext_mgr=ext_mgr,
            service_plugins=service_plugins)

        self.context = context.get_admin_context()
        self.internal_net_id = nsxv_db.get_nsxv_internal_network_for_az(
            self.context.session,
            vcns_const.InternalEdgePurposes.INTER_EDGE_PURPOSE,
            'default')['network_id']


class TestNetworksWithMdV2(test_plugin.TestNetworksV2,
                           NsxVPluginWithMdV2TestCase):

    # Skip all the tests that count networks, as there is an
    # additional internal network for metadata.
    def test_list_networks_with_sort_native(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_list_networks_without_pk_in_fields_pagination_emulated(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_list_networks_with_sort_emulated(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_list_networks_with_shared(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_list_networks_without_pk_in_fields_pagination_native(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_list_networks_with_parameters(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_list_networks_with_pagination_native(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_list_networks_with_pagination_reverse_emulated(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_list_networks(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_list_networks_with_pagination_emulated(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_list_networks_with_pagination_reverse_native(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_list_networks_with_fields(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_create_networks_bulk_wrong_input(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_create_networks_bulk_native_plugin_failure(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_create_networks_bulk_native_quotas(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_create_networks_bulk_emulated_plugin_failure(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_cannot_delete_md_net(self):
        req = self.new_delete_request('networks', self.internal_net_id)
        net_del_res = req.get_response(self.api)
        self.assertEqual(net_del_res.status_int, 400)


class TestSubnetsWithMdV2(test_plugin.TestSubnetsV2,
                          NsxVPluginWithMdV2TestCase):
    # Skip all the tests that count subnets, as there is an
    # additional internal subnet for metadata.
    def test_list_subnets_with_sort_native(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_list_subnets_with_sort_emulated(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_list_subnets_with_pagination_native(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_list_subnets_with_parameter(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_list_subnets_with_pagination_emulated(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_list_subnets_filtering_by_unknown_filter(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_list_subnets_shared(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_list_subnets(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_create_subnets_bulk_native_plugin_failure(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_create_subnets_bulk_native_quotas(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_create_subnets_bulk_emulated_plugin_failure(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_cannot_delete_md_subnet(self):
        query_params = "network_id=%s" % self.internal_net_id
        res = self._list('subnets',
                         neutron_context=self.context,
                         query_params=query_params)
        internal_sub = res['subnets'][0]['id']
        req = self.new_delete_request('subnets', internal_sub)
        net_del_res = req.get_response(self.api)
        self.assertEqual(net_del_res.status_int, 400)


class TestExclusiveRouterWithMdTestCase(
    test_plugin.TestExclusiveRouterTestCase,
    NsxVPluginWithMdV2TestCase):

    # Skip all the tests that count firewall rules, as there are
    # some MD specific rules
    def test_router_set_gateway_with_nosnat(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_router_interfaces_different_tenants_update_firewall(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_router_interfaces_with_update_firewall(self):
        self.skipTest("The test is not suitable for the metadata test case")

    # Skip all the tests that count routers or ports, as there is
    # an additional router for the md proxy
    def test_router_list_with_pagination_reverse(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_router_list_with_sort(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_router_list_with_pagination(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_router_list(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_router_add_interface_delete_port_after_failure(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_create_router_fail_at_the_backend(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_floatingip_delete_router_intf_with_subnet_id_returns_409(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_floatingip_delete_router_intf_with_port_id_returns_409(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_router_address_scope_snat_rules(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_router_address_scope_fw_rules(self):
        self.skipTest("The test is not suitable for the metadata test case")


class TestVdrWithMdTestCase(test_plugin.TestVdrTestCase,
                            NsxVPluginWithMdV2TestCase):
    # Skip all the tests that count firewall rules, as there are
    # some MD specific rules
    def test_router_set_gateway_with_nosnat(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_router_interfaces_different_tenants_update_firewall(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_router_interfaces_with_update_firewall(self):
        self.skipTest("The test is not suitable for the metadata test case")

    # Skip all the tests that count routers or ports, as there is
    # an additional router for the md proxy
    def test_router_list_with_pagination_reverse(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_router_list_with_sort(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_router_list_with_pagination(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_router_list(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_router_add_interface_delete_port_after_failure(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_create_router_fail_at_the_backend(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_floatingip_delete_router_intf_with_subnet_id_returns_409(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_floatingip_delete_router_intf_with_port_id_returns_409(self):
        self.skipTest("The test is not suitable for the metadata test case")

    #TODO(asarfaty): fix some mocks so those tests will pass
    def test_router_plr_binding_default_size(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_router_plr_binding_configured_size(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_router_plr_binding_default_az(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_router_plr_binding_with_az(self):
        self.skipTest("The test is not suitable for the metadata test case")


class TestSharedRouterWithMdTestCase(test_plugin.TestSharedRouterTestCase,
                                     NsxVPluginWithMdV2TestCase):
    # Skip all the tests that count firewall rules, as there are
    # some MD specific rules
    def test_router_set_gateway_with_nosnat(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_routers_set_gateway_with_nosnat(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_router_interfaces_different_tenants_update_firewall(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_router_interfaces_with_update_firewall(self):
        self.skipTest("The test is not suitable for the metadata test case")

    # Skip all the tests that count routers or ports, as there is
    # an additional router for the md proxy
    def test_router_list_with_pagination_reverse(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_router_list_with_sort(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_router_list_with_pagination(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_router_list(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_router_add_interface_delete_port_after_failure(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_create_router_fail_at_the_backend(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_floatingip_delete_router_intf_with_subnet_id_returns_409(self):
        self.skipTest("The test is not suitable for the metadata test case")

    def test_floatingip_delete_router_intf_with_port_id_returns_409(self):
        self.skipTest("The test is not suitable for the metadata test case")



# Copyright 2017 VMware, Inc.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

import mock
from neutron_lib.exceptions import firewall_v1 as exceptions
from oslo_utils import uuidutils

from vmware_nsx.services.fwaas.nsx_v import edge_fwaas_driver
from vmware_nsx.tests.unit.nsx_v import test_plugin as test_v_plugin

FAKE_FW_ID = 'fake_fw_uuid'


class NsxvFwaasTestCase(test_v_plugin.NsxVPluginV2TestCase):
    def setUp(self):
        super(NsxvFwaasTestCase, self).setUp()
        self.firewall = edge_fwaas_driver.EdgeFwaasDriver()

    def _fake_rules_v4(self):
        rule1 = {'enabled': True,
                 'action': 'allow',
                 'ip_version': 4,
                 'protocol': 'tcp',
                 'destination_port': '80',
                 'source_port': '1-65535',
                 'source_ip_address': '10.24.4.2',
                 'id': 'fake-fw-rule1'}
        rule2 = {'enabled': True,
                 'action': 'deny',
                 'ip_version': 4,
                 'protocol': 'tcp',
                 'destination_port': '22',
                 'id': 'fake-fw-rule2'}
        rule3 = {'enabled': True,
                 'action': 'reject',
                 'ip_version': 4,
                 'protocol': 'tcp',
                 'destination_port': '23',
                 'id': 'fake-fw-rule3'}
        return [rule1, rule2, rule3]

    def _fake_backend_rules_v4(self, logged=False):
        rule1 = {'enabled': True,
                 'action': 'allow',
                 'ip_version': 4,
                 'protocol': 'tcp',
                 'destination_port': '80',
                 'source_port': '1-65535',
                 'source_ip_address': ['10.24.4.2'],
                 'position': '0',
                 'id': 'fake-fw-rule1',
                 'name': 'Fwaas-fake-fw-rule1'}
        rule2 = {'enabled': True,
                 'action': 'deny',
                 'ip_version': 4,
                 'protocol': 'tcp',
                 'destination_port': '22',
                 'id': 'fake-fw-rule2',
                 'position': '1',
                 'name': 'Fwaas-fake-fw-rule2'}
        rule3 = {'enabled': True,
                 'action': 'reject',
                 'ip_version': 4,
                 'protocol': 'tcp',
                 'destination_port': '23',
                 'position': '2',
                 'id': 'fake-fw-rule3',
                 'name': 'Fwaas-fake-fw-rule3'}
        if logged:
            for rule in (rule1, rule2, rule3):
                rule['loggingEnabled'] = logged
        return [rule1, rule2, rule3]

    def _fake_firewall_no_rule(self):
        rule_list = []
        fw_inst = {'id': FAKE_FW_ID,
                   'admin_state_up': True,
                   'tenant_id': 'tenant-uuid',
                   'firewall_rule_list': rule_list}
        return fw_inst

    def _fake_firewall(self, rule_list):
        _rule_list = copy.deepcopy(rule_list)
        for rule in _rule_list:
            rule['position'] = str(_rule_list.index(rule))
        fw_inst = {'id': FAKE_FW_ID,
                   'admin_state_up': True,
                   'tenant_id': 'tenant-uuid',
                   'firewall_rule_list': _rule_list}
        return fw_inst

    def _fake_firewall_with_admin_down(self, rule_list):
        fw_inst = {'id': FAKE_FW_ID,
                   'admin_state_up': False,
                   'tenant_id': 'tenant-uuid',
                   'firewall_rule_list': rule_list}
        return fw_inst

    def _fake_apply_list(self, router_count=1):
        apply_list = []
        while router_count > 0:
            rtr_id = uuidutils.generate_uuid()
            router_inst = {'id': rtr_id}
            router_info_inst = mock.Mock()
            router_info_inst.router = router_inst
            router_info_inst.router_id = rtr_id
            apply_list.append(router_info_inst)
            router_count -= 1
        return apply_list

    def _get_fake_mapping(self, apply_list):
        router_edge_map = {}
        for router_info in apply_list:
            router_edge_map[router_info.router_id] = {
                'edge_id': 'edge-1',
                'lookup_id': router_info.router_id}
        return router_edge_map

    def _setup_firewall_with_rules(self, func, router_count=1):
        apply_list = self._fake_apply_list(router_count=router_count)
        rule_list = self._fake_rules_v4()
        firewall = self._fake_firewall(rule_list)
        edges = self._get_fake_mapping(apply_list)

        with mock.patch("vmware_nsx.plugins.nsx_v.plugin.NsxVPluginV2."
                        "update_router_firewall") as update_fw,\
            mock.patch("vmware_nsx.plugins.nsx_v.plugin.NsxVPluginV2."
                       "_get_router"),\
            mock.patch.object(self.firewall,
                              "_get_routers_edges", return_value=edges):
            func('nsx', apply_list, firewall)
            self.assertEqual(router_count, update_fw.call_count)
            # Validate the args of the last call
            self.assertEqual(apply_list[-1].router_id,
                             update_fw.call_args[0][1])
            backend_rules = update_fw.call_args[1]['fwaas_rules']
            self.assertEqual(len(rule_list), len(backend_rules))
            self.assertEqual(self._fake_backend_rules_v4(), backend_rules)

    def test_create_firewall_no_rules(self):
        apply_list = self._fake_apply_list()
        firewall = self._fake_firewall_no_rule()
        edges = self._get_fake_mapping(apply_list)
        with mock.patch("vmware_nsx.plugins.nsx_v.plugin.NsxVPluginV2."
                        "update_router_firewall") as update_fw,\
            mock.patch("vmware_nsx.plugins.nsx_v.plugin.NsxVPluginV2."
                       "_get_router"),\
            mock.patch.object(self.firewall,
                              "_get_routers_edges", return_value=edges):
            self.firewall.create_firewall('nsx', apply_list, firewall)
            self.assertEqual(1, update_fw.call_count)
            # Validate the args of the last call
            self.assertEqual(apply_list[0].router_id,
                             update_fw.call_args[0][1])
            backend_rules = update_fw.call_args[1]['fwaas_rules']
            self.assertEqual([], backend_rules)

    def test_create_firewall_with_rules(self):
        self._setup_firewall_with_rules(self.firewall.create_firewall)

    def test_create_firewall_with_rules_two_routers(self):
        self._setup_firewall_with_rules(self.firewall.create_firewall,
                                        router_count=2)

    def test_update_firewall_with_rules(self):
        self._setup_firewall_with_rules(self.firewall.update_firewall)

    def test_delete_firewall(self):
        apply_list = self._fake_apply_list()
        firewall = self._fake_firewall_no_rule()
        edges = self._get_fake_mapping(apply_list)
        with mock.patch("vmware_nsx.plugins.nsx_v.plugin.NsxVPluginV2."
                        "update_router_firewall") as update_fw,\
            mock.patch("vmware_nsx.plugins.nsx_v.plugin.NsxVPluginV2."
                       "_get_router"),\
            mock.patch.object(self.firewall,
                              "_get_routers_edges", return_value=edges):
            self.firewall.delete_firewall('nsx', apply_list, firewall)
            self.assertEqual(1, update_fw.call_count)
            # Validate the args of the last call
            self.assertEqual(apply_list[0].router_id,
                             update_fw.call_args[0][1])
            backend_rules = update_fw.call_args[1]['fwaas_rules']
            self.assertIsNone(backend_rules)

    def test_create_firewall_with_admin_down(self):
        apply_list = self._fake_apply_list()
        rule_list = self._fake_rules_v4()
        firewall = self._fake_firewall_with_admin_down(rule_list)
        edges = self._get_fake_mapping(apply_list)
        with mock.patch("vmware_nsx.plugins.nsx_v.plugin.NsxVPluginV2."
                        "update_router_firewall") as update_fw,\
            mock.patch("vmware_nsx.plugins.nsx_v.plugin.NsxVPluginV2."
                       "_get_router"),\
            mock.patch.object(self.firewall,
                              "_get_routers_edges", return_value=edges):
            self.firewall.create_firewall('nsx', apply_list, firewall)
            self.assertEqual(1, update_fw.call_count)
            # Validate the args of the last call
            self.assertEqual(apply_list[0].router_id,
                             update_fw.call_args[0][1])
            backend_rules = update_fw.call_args[1]['fwaas_rules']
            self.assertEqual([], backend_rules)

    def test_should_apply_firewall_to_router(self):
        router = {'id': 'fake_id',
                  'external_gateway_info': 'fake_data',
                  'router_type': 'exclusive',
                  'distributed': False}
        self.assertTrue(self.firewall.should_apply_firewall_to_router(router))

        # no external gateway:
        router['external_gateway_info'] = None
        self.assertFalse(self.firewall.should_apply_firewall_to_router(router))
        router['external_gateway_info'] = 'Dummy'

        # not for shared router:
        router['router_type'] = 'shared'
        router['distributed'] = False
        self.assertRaises(exceptions.FirewallInternalDriverError,
                          self.firewall.should_apply_firewall_to_router,
                          router)

        # should work for distributed router
        router['router_type'] = 'exclusive'
        router['distributed'] = True
        self.assertTrue(self.firewall.should_apply_firewall_to_router(router))

        # not for mdproxy router:
        router['name'] = 'metadata_proxy_router'
        self.assertRaises(exceptions.FirewallInternalDriverError,
                          self.firewall.should_apply_firewall_to_router,
                          router)
